#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils

from ironicclient.tests.functional.osc.v1 import base


class BaremetalPortCreate(base.TestCase):
    """Detailed functional tests for baremetal port create command."""

    def setUp(self):
        super(BaremetalPortCreate, self).setUp()
        self.node = self.node_create()

    def test_extras(self):
        """Check baremetal port create command with extra data.

        Test steps:
        1) Create port using solitary and multiple --extra arguments.
        2) Check that port successfully created with right extras.
        """
        extras = [{'single_extra': 'yes'},
                  {'k1': 'v1', 'k2': 'v2', 'k3': 'v3'}]

        for extra in extras:
            params = self.generate_params('--extra', extra)
            port = self.port_create(self.node['uuid'], params=params)
            self.assert_dict_is_subset(extra, port['extra'])

    def test_pxe_1_19(self):
        """Check baremetal port create command with PXE option.

        Test steps:
        1) Create port using --pxe-enabled argument.
        2) Check that port successfully created with right PXE option.
        """
        pxe_values = [True, False]
        api_version = ' --os-baremetal-api-version 1.19'

        for value in pxe_values:
            port = self.port_create(
                self.node['uuid'],
                params='--pxe-enabled {0} {1}'.format(value, api_version))
            self.assertEqual(value, port['pxe_enabled'])

    def test_llc_1_19(self):
        """Check baremetal port create command with LLC option.

        Test steps:
        1) Create port using --local-link-connection argument.
        2) Check that port successfully created with right LLC data.
        """
        fake_port_id = data_utils.rand_name(prefix='ovs-node-')
        fake_switch_id = data_utils.rand_mac_address()
        llc_value = {"switch_info": "brbm",
                     "port_id": fake_port_id,
                     "switch_id": fake_switch_id}
        api_version = ' --os-baremetal-api-version 1.19'

        params = self.generate_params('--local-link-connection', llc_value)
        port = self.port_create(self.node['uuid'],
                                params='{0} {1}'.format(params, api_version))
        self.assert_dict_is_subset(llc_value, port['local_link_connection'])



#  Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

import copy

from osc_lib.tests import utils as oscutils

from ironicclient.osc.v1 import baremetal_driver
from ironicclient.tests.unit.osc.v1 import fakes as baremetal_fakes


class TestBaremetalDriver(baremetal_fakes.TestBaremetal):

    def setUp(self):
        super(TestBaremetalDriver, self).setUp()

        self.baremetal_mock = self.app.client_manager.baremetal
        self.baremetal_mock.reset_mock()


class TestListBaremetalDriver(TestBaremetalDriver):

    def setUp(self):
        super(TestListBaremetalDriver, self).setUp()

        self.baremetal_mock.driver.list.return_value = [
            baremetal_fakes.FakeBaremetalResource(
                None,
                copy.deepcopy(baremetal_fakes.BAREMETAL_DRIVER),
                loaded=True)
        ]
        self.cmd = baremetal_driver.ListBaremetalDriver(self.app, None)

    def test_baremetal_driver_list(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        collist = (
            "Supported driver(s)",
            "Active host(s)")
        self.assertEqual(collist, tuple(columns))

        datalist = ((
            baremetal_fakes.baremetal_driver_name,
            ', '.join(baremetal_fakes.baremetal_driver_hosts)), )
        self.assertEqual(datalist, tuple(data))

    def test_baremetal_driver_list_with_type(self):
        arglist = ['--type', baremetal_fakes.baremetal_driver_type]
        verifylist = [('type', baremetal_fakes.baremetal_driver_type)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        collist = (
            "Supported driver(s)",
            "Active host(s)")
        self.assertEqual(collist, tuple(columns))

        datalist = ((
            baremetal_fakes.baremetal_driver_name,
            ', '.join(baremetal_fakes.baremetal_driver_hosts)),)
        self.assertEqual(datalist, tuple(data))

    def test_baremetal_driver_list_with_detail(self):
        arglist = ['--long']
        verifylist = [('long', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        collist = (
            "Supported driver(s)",
            "Type",
            "Active host(s)",
            'Default BIOS Interface',
            'Default Boot Interface',
            'Default Console Interface',
            'Default Deploy Interface',
            'Default Inspect Interface',
            'Default Management Interface',
            'Default Network Interface',
            'Default Power Interface',
            'Default RAID Interface',
            'Default Rescue Interface',
            'Default Storage Interface',
            'Default Vendor Interface',
            'Enabled BIOS Interfaces',
            'Enabled Boot Interfaces',
            'Enabled Console Interfaces',
            'Enabled Deploy Interfaces',
            'Enabled Inspect Interfaces',
            'Enabled Management Interfaces',
            'Enabled Network Interfaces',
            'Enabled Power Interfaces',
            'Enabled RAID Interfaces',
            'Enabled Rescue Interfaces',
            'Enabled Storage Interfaces',
            'Enabled Vendor Interfaces'
        )
        self.assertEqual(collist, tuple(columns))

        datalist = ((
            baremetal_fakes.baremetal_driver_name,
            baremetal_fakes.baremetal_driver_type,
            ', '.join(baremetal_fakes.baremetal_driver_hosts),
            baremetal_fakes.baremetal_driver_default_bios_if,
            baremetal_fakes.baremetal_driver_default_boot_if,
            baremetal_fakes.baremetal_driver_default_console_if,
            baremetal_fakes.baremetal_driver_default_deploy_if,
            baremetal_fakes.baremetal_driver_default_inspect_if,
            baremetal_fakes.baremetal_driver_default_management_if,
            baremetal_fakes.baremetal_driver_default_network_if,
            baremetal_fakes.baremetal_driver_default_power_if,
            baremetal_fakes.baremetal_driver_default_raid_if,
            baremetal_fakes.baremetal_driver_default_rescue_if,
            baremetal_fakes.baremetal_driver_default_storage_if,
            baremetal_fakes.baremetal_driver_default_vendor_if,
            ', '.join(baremetal_fakes.baremetal_driver_enabled_bios_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_boot_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_console_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_deploy_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_inspect_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_management_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_network_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_power_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_raid_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_rescue_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_storage_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_vendor_ifs),
        ),)
        self.assertEqual(datalist, tuple(data))

    def test_baremetal_driver_list_fields(self):
        arglist = [
            '--fields', 'name', 'hosts'
        ]
        verifylist = [
            ('fields', [['name', 'hosts']])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)

        kwargs = {
            'driver_type': None,
            'detail': None,
            'fields': ('name', 'hosts')
        }

        self.baremetal_mock.driver.list.assert_called_with(**kwargs)

    def test_baremetal_driver_list_fields_multiple(self):
        arglist = [
            '--fields', 'name',
            '--fields', 'hosts', 'type'
        ]
        verifylist = [
            ('fields', [['name'], ['hosts', 'type']])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)

        kwargs = {
            'driver_type': None,
            'detail': None,
            'fields': ('name', 'hosts', 'type')
        }

        self.baremetal_mock.driver.list.assert_called_with(**kwargs)

    def test_baremetal_driver_list_invalid_fields(self):
        arglist = [
            '--fields', 'name', 'invalid'
        ]
        verifylist = [
            ('fields', [['name', 'invalid']])
        ]
        self.assertRaises(oscutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_baremetal_driver_list_fields_with_long(self):
        arglist = [
            '--fields', 'name', 'hosts',
            '--long'
        ]
        verifylist = [
            ('fields', [['name', 'invalid']]),
            ('long', True)
        ]
        self.assertRaises(oscutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)


class TestListBaremetalDriverProperty(TestBaremetalDriver):

    def setUp(self):
        super(TestListBaremetalDriverProperty, self).setUp()

        self.baremetal_mock.driver.properties.return_value = {
            'property1': 'description1', 'property2': 'description2'}
        self.cmd = baremetal_driver.ListBaremetalDriverProperty(
            self.app, None)

    def test_baremetal_driver_property_list(self):
        arglist = ['fakedrivername']
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.baremetal_mock.driver.properties.assert_called_with(*arglist)

        collist = ['Property', 'Description']
        self.assertEqual(collist, columns)
        expected_data = [('property1', 'description1'),
                         ('property2', 'description2')]
        self.assertEqual(expected_data, data)

    def test_baremetal_driver_list_no_arg(self):
        arglist = []
        verifylist = []

        self.assertRaises(oscutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)


class TestListBaremetalDriverRaidProperty(TestBaremetalDriver):

    def setUp(self):
        super(TestListBaremetalDriverRaidProperty, self).setUp()

        (self.baremetal_mock.driver.
         raid_logical_disk_properties.return_value) = {
            'RAIDProperty1': 'driver_raid_property1',
            'RAIDProperty2': 'driver_raid_property2',
        }

        self.cmd = (
            baremetal_driver.ListBaremetalDriverRaidProperty(
                self.app, None))

    def test_baremetal_driver_raid_property_list(self):
        arglist = ['fakedrivername']
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        (self.baremetal_mock.driver.
         raid_logical_disk_properties.assert_called_with(*arglist))

        collist = ('Property', 'Description')
        self.assertEqual(collist, tuple(columns))

        expected_data = [('RAIDProperty1', 'driver_raid_property1'),
                         ('RAIDProperty2', 'driver_raid_property2')]
        self.assertEqual(expected_data, data)

    def test_baremetal_driver_raid_property_list_no_arg(self):
        arglist = []
        verifylist = []

        self.assertRaises(oscutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)


class TestPassthruCallBaremetalDriver(TestBaremetalDriver):

    def setUp(self):
        super(TestPassthruCallBaremetalDriver, self).setUp()

        self.baremetal_mock.driver.vendor_passthru.return_value = (
            baremetal_fakes.BAREMETAL_DRIVER_PASSTHRU
        )

        self.cmd = baremetal_driver.PassthruCallBaremetalDriver(self.app, None)

    def test_baremetal_driver_passthru_call_with_min_args(self):

        arglist = [
            baremetal_fakes.baremetal_driver_name,
            baremetal_fakes.baremetal_driver_passthru_method,
        ]

        verifylist = [
            ('driver', baremetal_fakes.baremetal_driver_name),
            ('method', baremetal_fakes.baremetal_driver_passthru_method),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)

        # Set expected values
        args = [
            baremetal_fakes.baremetal_driver_name,
            baremetal_fakes.baremetal_driver_passthru_method,
        ]
        kwargs = {
            'http_method': 'POST',
            'args': {}
        }
        (self.baremetal_mock.driver.vendor_passthru.
            assert_called_once_with(*args, **kwargs))

    def test_baremetal_driver_passthru_call_with_all_args(self):

        arglist = [
            baremetal_fakes.baremetal_driver_name,
            baremetal_fakes.baremetal_driver_passthru_method,
            '--arg', 'arg1=val1', '--arg', 'arg2=val2',
            '--http-method', 'POST'
        ]

        verifylist = [
            ('driver', baremetal_fakes.baremetal_driver_name),
            ('method', baremetal_fakes.baremetal_driver_passthru_method),
            ('arg', ['arg1=val1', 'arg2=val2']),
            ('http_method', 'POST')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)

        # Set expected values
        args = [
            baremetal_fakes.baremetal_driver_name,
            baremetal_fakes.baremetal_driver_passthru_method,
        ]
        kwargs = {
            'http_method': 'POST',
            'args': {'arg1': 'val1', 'arg2': 'val2'}
        }
        (self.baremetal_mock.driver.vendor_passthru.
            assert_called_once_with(*args, **kwargs))

    def test_baremetal_driver_passthru_call_no_arg(self):
        arglist = []
        verifylist = []

        self.assertRaises(oscutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)


class TestPassthruListBaremetalDriver(TestBaremetalDriver):

    def setUp(self):
        super(TestPassthruListBaremetalDriver, self).setUp()

        self.baremetal_mock.driver.get_vendor_passthru_methods.return_value = (
            baremetal_fakes.BAREMETAL_DRIVER_PASSTHRU
        )
        self.cmd = baremetal_driver.PassthruListBaremetalDriver(self.app, None)

    def test_baremetal_driver_passthru_list(self):
        arglist = ['fakedrivername']
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        args = ['fakedrivername']
        (self.baremetal_mock.driver.get_vendor_passthru_methods.
            assert_called_with(*args))

        collist = (
            "Name",
            "Supported HTTP methods",
            "Async",
            "Description",
            "Response is attachment",
        )
        self.assertEqual(collist, tuple(columns))

        datalist = (('lookup', 'POST', 'false', '', 'false'),)

        self.assertEqual(datalist, tuple(data))

    def test_baremetal_driver_passthru_list_no_arg(self):
        arglist = []
        verifylist = []

        self.assertRaises(oscutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)


class TestShowBaremetalDriver(TestBaremetalDriver):

    def setUp(self):
        super(TestShowBaremetalDriver, self).setUp()

        self.baremetal_mock.driver.get.return_value = (
            baremetal_fakes.FakeBaremetalResource(
                None,
                copy.deepcopy(baremetal_fakes.BAREMETAL_DRIVER),
                loaded=True))
        self.cmd = baremetal_driver.ShowBaremetalDriver(self.app, None)

    def test_baremetal_driver_show(self):
        arglist = ['fakedrivername']
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        args = ['fakedrivername']
        self.baremetal_mock.driver.get.assert_called_with(*args, fields=None)
        self.assertFalse(self.baremetal_mock.driver.properties.called)

        collist = ('default_bios_interface', 'default_boot_interface',
                   'default_console_interface', 'default_deploy_interface',
                   'default_inspect_interface',
                   'default_management_interface',
                   'default_network_interface', 'default_power_interface',
                   'default_raid_interface', 'default_rescue_interface',
                   'default_storage_interface', 'default_vendor_interface',
                   'enabled_bios_interfaces', 'enabled_boot_interfaces',
                   'enabled_console_interfaces', 'enabled_deploy_interfaces',
                   'enabled_inspect_interfaces',
                   'enabled_management_interfaces',
                   'enabled_network_interfaces', 'enabled_power_interfaces',
                   'enabled_raid_interfaces', 'enabled_rescue_interfaces',
                   'enabled_storage_interfaces', 'enabled_vendor_interfaces',
                   'hosts', 'name', 'type')
        self.assertEqual(collist, columns)

        datalist = (
            baremetal_fakes.baremetal_driver_default_bios_if,
            baremetal_fakes.baremetal_driver_default_boot_if,
            baremetal_fakes.baremetal_driver_default_console_if,
            baremetal_fakes.baremetal_driver_default_deploy_if,
            baremetal_fakes.baremetal_driver_default_inspect_if,
            baremetal_fakes.baremetal_driver_default_management_if,
            baremetal_fakes.baremetal_driver_default_network_if,
            baremetal_fakes.baremetal_driver_default_power_if,
            baremetal_fakes.baremetal_driver_default_raid_if,
            baremetal_fakes.baremetal_driver_default_rescue_if,
            baremetal_fakes.baremetal_driver_default_storage_if,
            baremetal_fakes.baremetal_driver_default_vendor_if,
            ', '.join(baremetal_fakes.baremetal_driver_enabled_bios_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_boot_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_console_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_deploy_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_inspect_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_management_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_network_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_power_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_raid_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_rescue_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_storage_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_enabled_vendor_ifs),
            ', '.join(baremetal_fakes.baremetal_driver_hosts),
            baremetal_fakes.baremetal_driver_name,
            baremetal_fakes.baremetal_driver_type)

        self.assertEqual(datalist, tuple(data))

    def test_baremetal_driver_show_no_arg(self):
        arglist = []
        verifylist = []

        self.assertRaises(oscutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_baremetal_driver_show_fields(self):
        arglist = [
            'fakedrivername',
            '--fields', 'name', 'hosts'
        ]
        verifylist = [
            ('driver', 'fakedrivername'),
            ('fields', [['name', 'hosts']])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        args = ['fakedrivername']
        fields = ['name', 'hosts']
        self.baremetal_mock.driver.get.assert_called_with(*args, fields=fields)
        self.assertFalse(self.baremetal_mock.driver.properties.called)

    def test_baremetal_driver_show_fields_multiple(self):
        arglist = [
            'fakedrivername',
            '--fields', 'name',
            '--fields', 'hosts', 'type'
        ]
        verifylist = [
            ('driver', 'fakedrivername'),
            ('fields', [['name'], ['hosts', 'type']])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        args = ['fakedrivername']
        fields = ['name', 'hosts', 'type']
        self.baremetal_mock.driver.get.assert_called_with(*args, fields=fields)
        self.assertFalse(self.baremetal_mock.driver.properties.called)

    def test_baremetal_driver_show_invalid_fields(self):
        arglist = [
            'fakedrivername',
            '--fields', 'name', 'invalid'
        ]
        verifylist = [
            ('driver', 'fakedrivername'),
            ('fields', [['name', 'invalid']])
        ]

        self.assertRaises(oscutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)



#
#   Copyright 2015 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import json
from unittest import mock

from osc_lib.tests import utils

from ironicclient.tests.unit.osc import fakes

baremetal_chassis_uuid = 'aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee'
baremetal_chassis_uuid_empty = ''
baremetal_properties_empty = ''
baremetal_chassis_description = 'chassis description'
baremetal_chassis_extra = {}
BAREMETAL_CHASSIS = {
    'uuid': baremetal_chassis_uuid,
    'description': baremetal_chassis_description,
    'extra': baremetal_chassis_extra,
}

baremetal_uuid = 'xxx-xxxxxx-xxxx'
baremetal_name = 'fake name'
baremetal_owner = 'fake-owner'
baremetal_instance_uuid = 'yyy-yyyyyy-yyyy'
baremetal_power_state = None
baremetal_provision_state = None
baremetal_maintenance = False

BAREMETAL = {
    'uuid': baremetal_uuid,
    'name': baremetal_name,
    'instance_uuid': baremetal_instance_uuid,
    'power_state': baremetal_power_state,
    'provision_state': baremetal_provision_state,
    'maintenance': baremetal_maintenance,
    'links': [],
    'volume': [],
}

baremetal_port_uuid = 'zzz-zzzzzz-zzzz'
baremetal_port_address = 'AA:BB:CC:DD:EE:FF'
baremetal_port_extra = {'key1': 'value1',
                        'key2': 'value2'}
baremetal_port_physical_network = 'physnet1'

BAREMETAL_PORT = {
    'uuid': baremetal_port_uuid,
    'address': baremetal_port_address,
    'extra': baremetal_port_extra,
    'node_uuid': baremetal_uuid,
}

baremetal_driver_hosts = ['fake-host1', 'fake-host2']
baremetal_driver_name = 'fakedrivername'
baremetal_driver_type = 'classic'
baremetal_driver_default_bios_if = 'bios'
baremetal_driver_default_boot_if = 'boot'
baremetal_driver_default_console_if = 'console'
baremetal_driver_default_deploy_if = 'deploy'
baremetal_driver_default_inspect_if = 'inspect'
baremetal_driver_default_management_if = 'management'
baremetal_driver_default_network_if = 'network'
baremetal_driver_default_power_if = 'power'
baremetal_driver_default_raid_if = 'raid'
baremetal_driver_default_rescue_if = 'rescue'
baremetal_driver_default_storage_if = 'storage'
baremetal_driver_default_vendor_if = 'vendor'
baremetal_driver_enabled_bios_ifs = ['bios', 'bios2']
baremetal_driver_enabled_boot_ifs = ['boot', 'boot2']
baremetal_driver_enabled_console_ifs = ['console', 'console2']
baremetal_driver_enabled_deploy_ifs = ['deploy', 'deploy2']
baremetal_driver_enabled_inspect_ifs = ['inspect', 'inspect2']
baremetal_driver_enabled_management_ifs = ['management', 'management2']
baremetal_driver_enabled_network_ifs = ['network', 'network2']
baremetal_driver_enabled_power_ifs = ['power', 'power2']
baremetal_driver_enabled_raid_ifs = ['raid', 'raid2']
baremetal_driver_enabled_rescue_ifs = ['rescue', 'rescue2']
baremetal_driver_enabled_storage_ifs = ['storage', 'storage2']
baremetal_driver_enabled_vendor_ifs = ['vendor', 'vendor2']

BAREMETAL_DRIVER = {
    'hosts': baremetal_driver_hosts,
    'name': baremetal_driver_name,
    'type': baremetal_driver_type,
    'default_bios_interface': baremetal_driver_default_bios_if,
    'default_boot_interface': baremetal_driver_default_boot_if,
    'default_console_interface': baremetal_driver_default_console_if,
    'default_deploy_interface': baremetal_driver_default_deploy_if,
    'default_inspect_interface': baremetal_driver_default_inspect_if,
    'default_management_interface': baremetal_driver_default_management_if,
    'default_network_interface': baremetal_driver_default_network_if,
    'default_power_interface': baremetal_driver_default_power_if,
    'default_raid_interface': baremetal_driver_default_raid_if,
    'default_rescue_interface': baremetal_driver_default_rescue_if,
    'default_storage_interface': baremetal_driver_default_storage_if,
    'default_vendor_interface': baremetal_driver_default_vendor_if,
    'enabled_bios_interfaces': baremetal_driver_enabled_bios_ifs,
    'enabled_boot_interfaces': baremetal_driver_enabled_boot_ifs,
    'enabled_console_interfaces': baremetal_driver_enabled_console_ifs,
    'enabled_deploy_interfaces': baremetal_driver_enabled_deploy_ifs,
    'enabled_inspect_interfaces': baremetal_driver_enabled_inspect_ifs,
    'enabled_management_interfaces': baremetal_driver_enabled_management_ifs,
    'enabled_network_interfaces': baremetal_driver_enabled_network_ifs,
    'enabled_power_interfaces': baremetal_driver_enabled_power_ifs,
    'enabled_raid_interfaces': baremetal_driver_enabled_raid_ifs,
    'enabled_rescue_interfaces': baremetal_driver_enabled_rescue_ifs,
    'enabled_storage_interfaces': baremetal_driver_enabled_storage_ifs,
    'enabled_vendor_interfaces': baremetal_driver_enabled_vendor_ifs,
}

baremetal_driver_passthru_method = 'lookup'

BAREMETAL_DRIVER_PASSTHRU = {"lookup": {"attach": "false",
                                        "http_methods": ["POST"],
                                        "description": "",
                                        "async": "false"}}

baremetal_portgroup_uuid = 'ppp-gggggg-pppp'
baremetal_portgroup_name = 'Portgroup-name'
baremetal_portgroup_address = 'AA:BB:CC:CC:BB:AA'
baremetal_portgroup_mode = 'active-backup'
baremetal_portgroup_extra = {'key1': 'value1',
                             'key2': 'value2'}
baremetal_portgroup_properties = {'key1': 'value11',
                                  'key2': 'value22'}

PORTGROUP = {'uuid': baremetal_portgroup_uuid,
             'name': baremetal_portgroup_name,
             'node_uuid': baremetal_uuid,
             'address': baremetal_portgroup_address,
             'extra': baremetal_portgroup_extra,
             'mode': baremetal_portgroup_mode,
             'properties': baremetal_portgroup_properties,
             }

VIFS = {'vifs': [{'id': 'aaa-aa'}]}
TRAITS = ['CUSTOM_FOO', 'CUSTOM_BAR']
BIOS_SETTINGS = [{'name': 'bios_name_1', 'value': 'bios_value_1', 'links': []},
                 {'name': 'bios_name_2', 'value': 'bios_value_2', 'links': []}]

BIOS_DETAILED_SETTINGS = [{'name': 'SysName', 'value': 'my-system',
                           'links': [], 'attribute_type': 'String',
                           'min_length': '1', 'max_length': '16'},
                          {'name': 'NumCores', 'value': '10',
                           'links': [], 'attribute_type': 'Integer',
                           'lower_bound': '10', 'upper_bound': '20'},
                          {'name': 'ProcVirtualization', 'value': 'Enabled',
                           'links': [], 'attribute_type': 'Enumeration',
                           'allowable_values': ['Enabled', 'Disabled']}]

baremetal_volume_connector_uuid = 'vvv-cccccc-vvvv'
baremetal_volume_connector_type = 'iqn'
baremetal_volume_connector_connector_id = 'iqn.2017-01.connector'
baremetal_volume_connector_extra = {'key1': 'value1',
                                    'key2': 'value2'}
VOLUME_CONNECTOR = {
    'uuid': baremetal_volume_connector_uuid,
    'node_uuid': baremetal_uuid,
    'type': baremetal_volume_connector_type,
    'connector_id': baremetal_volume_connector_connector_id,
    'extra': baremetal_volume_connector_extra,
}

baremetal_volume_target_uuid = 'vvv-tttttt-vvvv'
baremetal_volume_target_volume_type = 'iscsi'
baremetal_volume_target_boot_index = 0
baremetal_volume_target_volume_id = 'vvv-tttttt-iii'
baremetal_volume_target_extra = {'key1': 'value1',
                                 'key2': 'value2'}
baremetal_volume_target_properties = {'key11': 'value11',
                                      'key22': 'value22'}
VOLUME_TARGET = {
    'uuid': baremetal_volume_target_uuid,
    'node_uuid': baremetal_uuid,
    'volume_type': baremetal_volume_target_volume_type,
    'boot_index': baremetal_volume_target_boot_index,
    'volume_id': baremetal_volume_target_volume_id,
    'extra': baremetal_volume_target_extra,
    'properties': baremetal_volume_target_properties,
}

baremetal_hostname = 'compute1.localdomain'
baremetal_conductor_group = 'foo'
baremetal_alive = True
baremetal_drivers = ['fake-hardware']
CONDUCTOR = {
    'hostname': baremetal_hostname,
    'conductor_group': baremetal_conductor_group,
    'alive': baremetal_alive,
    'drivers': baremetal_drivers,
}

baremetal_allocation_state = 'active'
baremetal_resource_class = 'baremetal'
ALLOCATION = {
    'resource_class': baremetal_resource_class,
    'uuid': baremetal_uuid,
    'name': baremetal_name,
    'state': baremetal_allocation_state,
    'node_uuid': baremetal_uuid,
}

baremetal_deploy_template_uuid = 'ddd-tttttt-dddd'
baremetal_deploy_template_name = 'DeployTemplate-name'
baremetal_deploy_template_steps = json.dumps([{
    'interface': 'raid',
    'step': 'create_configuration',
    'args': {},
    'priority': 10
}])
baremetal_deploy_template_extra = {'key1': 'value1', 'key2': 'value2'}
DEPLOY_TEMPLATE = {
    'uuid': baremetal_deploy_template_uuid,
    'name': baremetal_deploy_template_name,
    'steps': baremetal_deploy_template_steps,
    'extra': baremetal_deploy_template_extra,
}


class TestBaremetal(utils.TestCommand):

    def setUp(self):
        super(TestBaremetal, self).setUp()

        self.app.client_manager.auth_ref = mock.Mock(auth_token="TOKEN")
        self.app.client_manager.baremetal = mock.Mock()


class FakeBaremetalResource(fakes.FakeResource):

    def get_keys(self):
        return {'property': 'value'}



# Copyright 2012 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import argparse
import base64
import contextlib
import gzip
import json
import os
import shutil
import subprocess
import sys
import tempfile
import time

from oslo_utils import strutils
import yaml

from ironicclient.common.i18n import _
from ironicclient import exc


class HelpFormatter(argparse.HelpFormatter):
    def start_section(self, heading):
        super(HelpFormatter, self).start_section(heading.capitalize())


def define_command(subparsers, command, callback, cmd_mapper):
    """Define a command in the subparsers collection.

    :param subparsers: subparsers collection where the command will go
    :param command: command name
    :param callback: function that will be used to process the command
    """
    desc = callback.__doc__ or ''
    help = desc.strip().split('\n')[0]
    arguments = getattr(callback, 'arguments', [])

    subparser = subparsers.add_parser(command, help=help,
                                      description=desc,
                                      add_help=False,
                                      formatter_class=HelpFormatter)
    subparser.add_argument('-h', '--help', action='help',
                           help=argparse.SUPPRESS)
    cmd_mapper[command] = subparser
    required_args = subparser.add_argument_group(_("Required arguments"))

    for (args, kwargs) in arguments:
        if kwargs.get('required'):
            required_args.add_argument(*args, **kwargs)
        else:
            subparser.add_argument(*args, **kwargs)
    subparser.set_defaults(func=callback)


def define_commands_from_module(subparsers, command_module, cmd_mapper):
    """Add *do_* methods in a module and add as commands into a subparsers."""

    for method_name in (a for a in dir(command_module) if a.startswith('do_')):
        # Commands should be hypen-separated instead of underscores.
        command = method_name[3:].replace('_', '-')
        callback = getattr(command_module, method_name)
        define_command(subparsers, command, callback, cmd_mapper)


def split_and_deserialize(string):
    """Split and try to JSON deserialize a string.

    Gets a string with the KEY=VALUE format, split it (using '=' as the
    separator) and try to JSON deserialize the VALUE.

    :returns: A tuple of (key, value).
    """
    try:
        key, value = string.split("=", 1)
    except ValueError:
        raise exc.CommandError(_('Attributes must be a list of '
                                 'PATH=VALUE not "%s"') % string)
    try:
        value = json.loads(value)
    except ValueError:
        pass

    return (key, value)


def key_value_pairs_to_dict(key_value_pairs):
    """Convert a list of key-value pairs to a dictionary.

    :param key_value_pairs: a list of strings, each string is in the form
                            <key>=<value>
    :returns: a dictionary, possibly empty
    """
    if key_value_pairs:
        return dict(split_and_deserialize(v) for v in key_value_pairs)
    return {}


def args_array_to_dict(kwargs, key_to_convert):
    """Convert the value in a dictionary entry to a dictionary.

    From the kwargs dictionary, converts the value of the key_to_convert
    entry from a list of key-value pairs to a dictionary.

    :param kwargs: a dictionary
    :param key_to_convert: the key (in kwargs), whose value is expected to
        be a list of key=value strings. This value will be converted to a
        dictionary.
    :returns: kwargs, the (modified) dictionary
    """
    values_to_convert = kwargs.get(key_to_convert)
    if values_to_convert:
        kwargs[key_to_convert] = key_value_pairs_to_dict(values_to_convert)
    return kwargs


def args_array_to_patch(op, attributes):
    patch = []
    for attr in attributes:
        # Sanitize
        if not attr.startswith('/'):
            attr = '/' + attr

        if op in ['add', 'replace']:
            path, value = split_and_deserialize(attr)
            patch.append({'op': op, 'path': path, 'value': value})

        elif op == "remove":
            # For remove only the key is needed
            patch.append({'op': op, 'path': attr})
        else:
            raise exc.CommandError(_('Unknown PATCH operation: %s') % op)
    return patch


def convert_list_props_to_comma_separated(data, props=None):
    """Convert the list-type properties to comma-separated strings

    :param data: the input dict object.
    :param props: the properties whose values will be converted.
        Default to None to convert all list-type properties of the input.
    :returns: the result dict instance.
    """
    result = dict(data)

    if props is None:
        props = data.keys()

    for prop in props:
        val = data.get(prop, None)
        if isinstance(val, list):
            result[prop] = ', '.join(map(str, val))

    return result


def common_params_for_list(args, fields, field_labels):
    """Generate 'params' dict that is common for every 'list' command.

    :param args: arguments from command line.
    :param fields: possible fields for sorting.
    :param field_labels: possible field labels for sorting.
    :returns: a dict with params to pass to the client method.
    """
    params = {}
    if args.marker is not None:
        params['marker'] = args.marker
    if args.limit is not None:
        if args.limit < 0:
            raise exc.CommandError(
                _('Expected non-negative --limit, got %s') % args.limit)
        params['limit'] = args.limit

    if args.sort_key is not None:
        # Support using both heading and field name for sort_key
        fields_map = dict(zip(field_labels, fields))
        fields_map.update(zip(fields, fields))
        try:
            sort_key = fields_map[args.sort_key]
        except KeyError:
            raise exc.CommandError(
                _("%(sort_key)s is an invalid field for sorting, "
                  "valid values for --sort-key are: %(valid)s") %
                {'sort_key': args.sort_key,
                 'valid': list(fields_map)})
        params['sort_key'] = sort_key
    if args.sort_dir is not None:
        if args.sort_dir not in ('asc', 'desc'):
            raise exc.CommandError(
                _("%s is an invalid value for sort direction, "
                  "valid values for --sort-dir are: 'asc', 'desc'") %
                args.sort_dir)
        params['sort_dir'] = args.sort_dir

    params['detail'] = args.detail

    requested_fields = args.fields[0] if args.fields else None
    if requested_fields is not None:
        params['fields'] = requested_fields

    return params


def common_filters(marker=None, limit=None, sort_key=None, sort_dir=None,
                   fields=None, detail=False):
    """Generate common filters for any list request.

    :param marker: entity ID from which to start returning entities.
    :param limit: maximum number of entities to return.
    :param sort_key: field to use for sorting.
    :param sort_dir: direction of sorting: 'asc' or 'desc'.
    :param fields: a list with a specified set of fields of the resource
                   to be returned.
    :param detail: Boolean, True to return detailed information. This parameter
                   can be used for resources which accept 'detail' as a URL
                   parameter.
    :returns: list of string filters.
    """
    filters = []
    if isinstance(limit, int) and limit > 0:
        filters.append('limit=%s' % limit)
    if marker is not None:
        filters.append('marker=%s' % marker)
    if sort_key is not None:
        filters.append('sort_key=%s' % sort_key)
    if sort_dir is not None:
        filters.append('sort_dir=%s' % sort_dir)
    if fields is not None:
        filters.append('fields=%s' % ','.join(fields))
    if detail:
        filters.append('detail=True')
    return filters


@contextlib.contextmanager
def tempdir(*args, **kwargs):
    dirname = tempfile.mkdtemp(*args, **kwargs)
    try:
        yield dirname
    finally:
        shutil.rmtree(dirname)


def make_configdrive(path):
    """Make the config drive file.

    :param path: The directory containing the config drive files.
    :returns: A gzipped and base64 encoded configdrive string.

    """
    # Make sure path it's readable
    if not os.access(path, os.R_OK):
        raise exc.CommandError(_('The directory "%s" is not readable') % path)

    with tempfile.NamedTemporaryFile() as tmpfile:
        with tempfile.NamedTemporaryFile() as tmpzipfile:
            publisher = 'ironicclient-configdrive 0.1'
            # NOTE(toabctl): Luckily, genisoimage, mkisofs and xorrisofs
            # understand the same parameters which are currently used.
            cmds = ['genisoimage', 'mkisofs', 'xorrisofs']
            for c in cmds:
                try:
                    p = subprocess.Popen([c, '-o', tmpfile.name,
                                          '-ldots', '-allow-lowercase',
                                          '-allow-multidot', '-l',
                                          '-publisher', publisher,
                                          '-quiet', '-J',
                                          '-r', '-V', 'config-2',
                                          path],
                                         stdout=subprocess.PIPE,
                                         stderr=subprocess.PIPE)
                except OSError as e:
                    error = e
                else:
                    error = None
                    break
            if error:
                raise exc.CommandError(
                    _('Error generating the config drive. Make sure the '
                      '"genisoimage", "mkisofs", or "xorriso" tool is '
                      'installed. Error: %s') % error)

            stdout, stderr = p.communicate()
            if p.returncode != 0:
                raise exc.CommandError(
                    _('Error generating the config drive.'
                      'Stdout: "%(stdout)s". Stderr: %(stderr)s') %
                    {'stdout': stdout, 'stderr': stderr})

            # Compress file
            tmpfile.seek(0)
            with gzip.GzipFile(fileobj=tmpzipfile, mode='wb') as gz_file:
                shutil.copyfileobj(tmpfile, gz_file)

            tmpzipfile.seek(0)
            return base64.b64encode(tmpzipfile.read())


def check_empty_arg(arg, arg_descriptor):
    if not arg.strip():
        raise exc.CommandError(_('%(arg)s cannot be empty or only have blank'
                                 ' spaces') % {'arg': arg_descriptor})


def bool_argument_value(arg_name, bool_str, strict=True, default=False):
    """Returns the Boolean represented by bool_str.

    Returns the Boolean value for the argument named arg_name. The value is
    represented by the string bool_str. If the string is an invalid Boolean
    string: if strict is True, a CommandError exception is raised; otherwise
    the default value is returned.

    :param arg_name: The name of the argument
    :param bool_str: The string representing a Boolean value
    :param strict: Used if the string is invalid. If True, raises an exception.
        If False, returns the default value.
    :param default: The default value to return if the string is invalid
        and not strict
    :returns: the Boolean value represented by bool_str or the default value
        if bool_str is invalid and strict is False
    :raises CommandError: if bool_str is an invalid Boolean string

    """
    try:
        val = strutils.bool_from_string(bool_str, strict, default)
    except ValueError as e:
        raise exc.CommandError(_("argument %(arg)s: %(err)s.")
                               % {'arg': arg_name, 'err': e})
    return val


def check_for_invalid_fields(fields, valid_fields):
    """Check for invalid fields.

    :param fields: A list of fields specified by the user.
    :param valid_fields: A list of valid fields.
    :raises CommandError: If invalid fields were specified by the user.
    """
    if not fields:
        return

    invalid_fields = set(fields) - set(valid_fields)
    if invalid_fields:
        raise exc.CommandError(
            _('Invalid field(s) requested: %(invalid)s. Valid fields '
              'are: %(valid)s.') % {'invalid': ', '.join(invalid_fields),
                                    'valid': ', '.join(valid_fields)})


def get_from_stdin(info_desc):
    """Read information from stdin.

    :param info_desc: A string description of the desired information
    :raises: InvalidAttribute if there was a problem reading from stdin
    :returns: the string that was read from stdin
    """
    try:
        info = sys.stdin.read().strip()
    except Exception as e:
        err = _("Cannot get %(desc)s from standard input. Error: %(err)s")
        raise exc.InvalidAttribute(err % {'desc': info_desc, 'err': e})
    return info


def handle_json_or_file_arg(json_arg):
    """Attempts to read JSON argument from file or string.

    :param json_arg: May be a file name containing the YAML or JSON, or
        a JSON string.
    :returns: A list or dictionary parsed from JSON.
    :raises: InvalidAttribute if the argument cannot be parsed.
    """

    if os.path.isfile(json_arg):
        try:
            with open(json_arg, 'r') as f:
                return yaml.safe_load(f)
        except Exception as e:
            err = _("Cannot get JSON/YAML from file '%(file)s'. "
                    "Error: %(err)s") % {'err': e, 'file': json_arg}
            raise exc.InvalidAttribute(err)
    try:
        json_arg = json.loads(json_arg)
    except ValueError as e:
        err = (_("Value '%(string)s' is not a file and cannot be parsed "
                 "as JSON: '%(err)s'") % {'err': e, 'string': json_arg})
        raise exc.InvalidAttribute(err)

    return json_arg


def poll(timeout, poll_interval, poll_delay_function, timeout_message):
    if not isinstance(timeout, (int, float)) or timeout < 0:
        raise ValueError(_('Timeout must be a non-negative number'))

    threshold = time.time() + timeout
    poll_delay_function = (time.sleep if poll_delay_function is None
                           else poll_delay_function)
    if not callable(poll_delay_function):
        raise TypeError(_('poll_delay_function must be callable'))

    count = 0
    while not timeout or time.time() < threshold:
        yield count

        poll_delay_function(poll_interval)
        count += 1

    raise exc.StateTransitionTimeout(timeout_message)


def handle_json_arg(json_arg, info_desc):
    """Read a JSON argument from stdin, file or string.

    :param json_arg: May be a file name containing the JSON, a JSON string, or
        '-' indicating that the argument should be read from standard input.
    :param info_desc: A string description of the desired information
    :returns: A list or dictionary parsed from JSON.
    :raises: InvalidAttribute if the argument cannot be parsed.
    """
    if json_arg == '-':
        json_arg = get_from_stdin(info_desc)
    if json_arg:
        json_arg = handle_json_or_file_arg(json_arg)
    return json_arg



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ironicclient.tests.functional.osc.v1 import base


class BaremetalChassisTests(base.TestCase):
    """Functional tests for baremetal chassis commands."""

    def setUp(self):
        super(BaremetalChassisTests, self).setUp()
        self.chassis = self.chassis_create()

    def test_list(self):
        """Check baremetal chassis list command.

        Test steps:
        1) Create baremetal chassis in setUp.
        2) List baremetal chassis.
        3) Check chassis description and UUID in chassis list.
        """
        chassis_list = self.chassis_list()
        self.assertIn(self.chassis['uuid'],
                      [x['UUID'] for x in chassis_list])
        self.assertIn(self.chassis['description'],
                      [x['Description'] for x in chassis_list])

    def test_show(self):
        """Check baremetal chassis show command.

        Test steps:
        1) Create baremetal chassis in setUp.
        2) Show baremetal chassis.
        3) Check chassis in chassis show.
        """
        chassis = self.chassis_show(self.chassis['uuid'])
        self.assertEqual(self.chassis['uuid'], chassis['uuid'])
        self.assertEqual(self.chassis['description'], chassis['description'])

    def test_delete(self):
        """Check baremetal chassis delete command.

        Test steps:
        1) Create baremetal chassis in setUp.
        2) Delete baremetal chassis by UUID.
        3) Check that chassis deleted successfully.
        """
        output = self.chassis_delete(self.chassis['uuid'])
        self.assertIn('Deleted chassis {0}'.format(self.chassis['uuid']),
                      output)
        self.assertNotIn(self.chassis['uuid'], self.chassis_list(['UUID']))

    def test_set_unset_extra(self):
        """Check baremetal chassis set and unset commands.

        Test steps:
        1) Create baremetal chassis in setUp.
        2) Set extra data for chassis.
        3) Check that baremetal chassis extra data was set.
        4) Unset extra data for chassis.
        5) Check that baremetal chassis extra data was unset.
        """
        extra_key = 'ext'
        extra_value = 'testdata'
        self.openstack('baremetal chassis set --extra {0}={1} {2}'
                       .format(extra_key, extra_value, self.chassis['uuid']))

        show_prop = self.chassis_show(self.chassis['uuid'], ['extra'])
        self.assertEqual(extra_value, show_prop['extra'][extra_key])

        self.openstack('baremetal chassis unset --extra {0} {1}'
                       .format(extra_key, self.chassis['uuid']))

        show_prop = self.chassis_show(self.chassis['uuid'], ['extra'])
        self.assertNotIn(extra_key, show_prop['extra'])



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslotest import base as test_base

from ironicclient.common.apiclient import base


class HumanResource(base.Resource):
    HUMAN_ID = True


class HumanResourceManager(base.ManagerWithFind):
    resource_class = HumanResource

    def list(self):
        return self._list("/human_resources", "human_resources")

    def get(self, human_resource):
        return self._get(
            "/human_resources/%s" % base.getid(human_resource),
            "human_resource")

    def update(self, human_resource, name):
        body = {
            "human_resource": {
                "name": name,
            },
        }
        return self._put(
            "/human_resources/%s" % base.getid(human_resource),
            body,
            "human_resource")


class CrudResource(base.Resource):
    pass


class CrudResourceManager(base.CrudManager):
    """Manager class for manipulating Identity crud_resources."""
    resource_class = CrudResource
    collection_key = 'crud_resources'
    key = 'crud_resource'

    def get(self, crud_resource):
        return super(CrudResourceManager, self).get(
            crud_resource_id=base.getid(crud_resource))


class ResourceTest(test_base.BaseTestCase):
    def test_resource_repr(self):
        r = base.Resource(None, dict(foo="bar", baz="spam"))
        self.assertEqual("<Resource baz=spam, foo=bar>", repr(r))

    def test_getid(self):
        class TmpObject(base.Resource):
            id = "4"
        self.assertEqual("4", base.getid(TmpObject(None, {})))

    def test_human_id(self):
        r = base.Resource(None, {"name": "1"})
        self.assertIsNone(r.human_id)
        r = HumanResource(None, {"name": "1"})
        self.assertEqual("1", r.human_id)
        r = HumanResource(None, {"name": None})
        self.assertIsNone(r.human_id)

    def test_two_resources_with_same_id_are_not_equal(self):
        # Two resources with same ID: never equal if their info is not equal
        r1 = base.Resource(None, {'id': 1, 'name': 'hi'})
        r2 = base.Resource(None, {'id': 1, 'name': 'hello'})
        self.assertNotEqual(r1, r2)

    def test_two_resources_with_same_id_and_info_are_equal(self):
        # Two resources with same ID: equal if their info is equal
        r1 = base.Resource(None, {'id': 1, 'name': 'hello'})
        r2 = base.Resource(None, {'id': 1, 'name': 'hello'})
        self.assertEqual(r1, r2)

    def test_two_resources_with_diff_type_are_not_equal(self):
        # Two resoruces of different types: never equal
        r1 = base.Resource(None, {'id': 1})
        r2 = HumanResource(None, {'id': 1})
        self.assertNotEqual(r1, r2)

    def test_two_resources_with_no_id_are_equal(self):
        # Two resources with no ID: equal if their info is equal
        r1 = base.Resource(None, {'name': 'joe', 'age': 12})
        r2 = base.Resource(None, {'name': 'joe', 'age': 12})
        self.assertEqual(r1, r2)


class BaseManagerTestCase(test_base.BaseTestCase):

    def setUp(self):
        super(BaseManagerTestCase, self).setUp()

        self.response = mock.MagicMock()
        self.http_client = mock.MagicMock()
        self.http_client.get.return_value = self.response
        self.http_client.post.return_value = self.response

        self.manager = base.BaseManager(self.http_client)
        self.manager.resource_class = HumanResource

    def test_list(self):
        self.response.json.return_value = {'human_resources': [{'id': 42}]}
        expected = [HumanResource(self.manager, {'id': 42}, loaded=True)]
        result = self.manager._list("/human_resources", "human_resources")
        self.assertEqual(expected, result)

    def test_list_no_response_key(self):
        self.response.json.return_value = [{'id': 42}]
        expected = [HumanResource(self.manager, {'id': 42}, loaded=True)]
        result = self.manager._list("/human_resources")
        self.assertEqual(expected, result)

    def test_list_get(self):
        self.manager._list("/human_resources", "human_resources")
        self.manager.client.get.assert_called_with("/human_resources")

    def test_list_post(self):
        self.manager._list("/human_resources", "human_resources",
                           json={'id': 42})
        self.manager.client.post.assert_called_with("/human_resources",
                                                    json={'id': 42})

    def test_get(self):
        self.response.json.return_value = {'human_resources': {'id': 42}}
        expected = HumanResource(self.manager, {'id': 42}, loaded=True)
        result = self.manager._get("/human_resources/42", "human_resources")
        self.manager.client.get.assert_called_with("/human_resources/42")
        self.assertEqual(expected, result)

    def test_get_no_response_key(self):
        self.response.json.return_value = {'id': 42}
        expected = HumanResource(self.manager, {'id': 42}, loaded=True)
        result = self.manager._get("/human_resources/42")
        self.manager.client.get.assert_called_with("/human_resources/42")
        self.assertEqual(expected, result)

    def test_post(self):
        self.response.json.return_value = {'human_resources': {'id': 42}}
        expected = HumanResource(self.manager, {'id': 42}, loaded=True)
        result = self.manager._post("/human_resources",
                                    response_key="human_resources",
                                    json={'id': 42})
        self.manager.client.post.assert_called_with("/human_resources",
                                                    json={'id': 42})
        self.assertEqual(expected, result)

    def test_post_return_raw(self):
        self.response.json.return_value = {'human_resources': {'id': 42}}
        result = self.manager._post("/human_resources",
                                    response_key="human_resources",
                                    json={'id': 42}, return_raw=True)
        self.manager.client.post.assert_called_with("/human_resources",
                                                    json={'id': 42})
        self.assertEqual({'id': 42}, result)

    def test_post_no_response_key(self):
        self.response.json.return_value = {'id': 42}
        expected = HumanResource(self.manager, {'id': 42}, loaded=True)
        result = self.manager._post("/human_resources", json={'id': 42})
        self.manager.client.post.assert_called_with("/human_resources",
                                                    json={'id': 42})
        self.assertEqual(expected, result)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from ironicclient.tests.functional import base


class TestCase(base.FunctionalTestBase):

    driver_name = 'fake-hardware'

    def openstack(self, *args, **kwargs):
        return self._ironic(cmd='openstack', *args, **kwargs)

    def get_opts(self, fields=None, output_format='json'):
        """Get options for OSC output fields format.

        :param List fields: List of fields to get
        :param String output_format: Select output format
        :return: String of formatted options
        """
        if not fields:
            return ' -f {0}'.format(output_format)
        return ' -f {0} {1}'.format(output_format,
                                    ' '.join(['-c ' + it for it in fields]))

    @staticmethod
    def construct_cmd(*parts):
        return ' '.join(str(x) for x in parts)

    @staticmethod
    def generate_params(argument, params):
        """Generate parameters string.

        :param argument: argument
        :param params: values passed with argument
        """
        parts = []
        for key, value in params.items():
            parts.append('{} {}={}'.format(argument, key, value))
        return ' '.join(parts)

    def assert_dict_is_subset(self, expected, actual):
        """Check if expected keys/values exist in actual response body.

        Check if the expected keys and values are in the actual response body.

        :param expected: dict of key-value pairs that are expected to be in
                         'actual' dict.
        :param actual: dict of key-value pairs.
        """
        for key, value in expected.items():
            self.assertEqual(value, actual[key])

    def node_create(self, driver=driver_name, name=None, params=''):
        """Create baremetal node and add cleanup.

        :param String driver: Driver for a new node
        :param String name: Name for a new node
        :param String params: Additional args and kwargs
        :return: JSON object of created node
        """
        if not name:
            name = data_utils.rand_name('baremetal')

        opts = self.get_opts()
        output = self.openstack('baremetal node create {0} '
                                '--driver {1} --name {2} {3}'
                                .format(opts, driver, name, params))
        node = json.loads(output)
        self.addCleanup(self.node_delete, node['uuid'], True)
        if not output:
            self.fail('Baremetal node has not been created!')

        return node

    def node_list(self, fields=None, params=''):
        """List baremetal nodes.

        :param List fields: List of fields to show
        :param String params: Additional kwargs
        :return: list of JSON node objects
        """
        opts = self.get_opts(fields=fields)
        output = self.openstack('baremetal node list {0} {1}'
                                .format(opts, params))
        return json.loads(output)

    def node_show(self, identifier, fields=None, params=''):
        """Show specified baremetal node.

        :param String identifier: Name or UUID of the node
        :param List fields: List of fields to show
        :param List params: Additional kwargs
        :return: JSON object of node
        """
        opts = self.get_opts(fields)
        output = self.openstack('baremetal node show {0} {1} {2}'
                                .format(opts, identifier, params))
        return json.loads(output)

    def node_delete(self, identifier, ignore_exceptions=False):
        """Try to delete baremetal node by name or UUID.

        :param String identifier: Name or UUID of the node
        :param Bool ignore_exceptions: Ignore exception (needed for cleanUp)
        :return: raw values output
        :raise: CommandFailed exception when command fails to delete a node
        """
        try:
            return self.openstack('baremetal node delete {0}'
                                  .format(identifier))
        except exceptions.CommandFailed:
            if not ignore_exceptions:
                raise

    def port_create(self, node_id, mac_address=None, params=''):
        """Create baremetal port and add cleanup.

        :param String node_id: baremetal node UUID
        :param String mac_address: MAC address for port
        :param String params: Additional args and kwargs
        :return: JSON object of created port
        """
        if not mac_address:
            mac_address = data_utils.rand_mac_address()

        opts = self.get_opts()
        port = self.openstack('baremetal port create {0} '
                              '--node {1} {2} {3}'
                              .format(opts, node_id, mac_address, params))
        port = json.loads(port)
        if not port:
            self.fail('Baremetal port has not been created!')
        self.addCleanup(self.port_delete, port['uuid'], True)
        return port

    def port_list(self, fields=None, params=''):
        """List baremetal ports.

        :param List fields: List of fields to show
        :param String params: Additional kwargs
        :return: list of JSON port objects
        """
        opts = self.get_opts(fields=fields)
        output = self.openstack('baremetal port list {0} {1}'
                                .format(opts, params))
        return json.loads(output)

    def port_show(self, uuid, fields=None, params=''):
        """Show specified baremetal port.

        :param String uuid: UUID of the port
        :param List fields: List of fields to show
        :param List params: Additional kwargs
        :return: JSON object of port
        """
        opts = self.get_opts(fields)
        output = self.openstack('baremetal port show {0} {1} {2}'
                                .format(opts, uuid, params))
        return json.loads(output)

    def port_delete(self, uuid, ignore_exceptions=False):
        """Try to delete baremetal port by UUID.

        :param String uuid: UUID of the port
        :param Bool ignore_exceptions: Ignore exception (needed for cleanUp)
        :return: raw values output
        :raise: CommandFailed exception when command fails to delete a port
        """
        try:
            return self.openstack('baremetal port delete {0}'
                                  .format(uuid))
        except exceptions.CommandFailed:
            if not ignore_exceptions:
                raise

    def port_group_list(self, fields=None, params=''):
        """List baremetal port groups.

        :param List fields: List of fields to show
        :param String params: Additional kwargs
        :return: JSON object of port group list
        """
        opts = self.get_opts(fields=fields)
        output = self.openstack('baremetal port group list {0} {1}'
                                .format(opts, params))
        return json.loads(output)

    def port_group_create(self, node_id, name=None, params=''):
        """Create baremetal port group.

        :param String node_id: baremetal node UUID
        :param String name: port group name
        :param String params: Additional args and kwargs
        :return: JSON object of created port group
        """
        if not name:
            name = data_utils.rand_name('port_group')

        opts = self.get_opts()
        output = self.openstack(
            'baremetal port group create {0} --node {1} --name {2} {3}'
            .format(opts, node_id, name, params))

        port_group = json.loads(output)
        if not port_group:
            self.fail('Baremetal port group has not been created!')

        self.addCleanup(self.port_group_delete, port_group['uuid'],
                        params=params, ignore_exceptions=True)
        return port_group

    def port_group_delete(self, identifier, params='',
                          ignore_exceptions=False):
        """Try to delete baremetal port group by Name or UUID.

        :param String identifier: Name or UUID of the port group
        :param String params: temporary arg to pass api version.
        :param Bool ignore_exceptions: Ignore exception (needed for cleanUp)
        :return: raw values output
        :raise: CommandFailed exception if not ignore_exceptions
        """
        try:
            return self.openstack('baremetal port group delete {0} {1}'
                                  .format(identifier, params))
        except exceptions.CommandFailed:
            if not ignore_exceptions:
                raise

    def port_group_show(self, identifier, fields=None, params=''):
        """Show specified baremetal port group.

        :param String identifier: Name or UUID of the port group
        :param List fields: List of fields to show
        :param List params: Additional kwargs
        :return: JSON object of port group
        """
        opts = self.get_opts(fields)
        output = self.openstack('baremetal port group show {0} {1} {2}'
                                .format(identifier, opts, params))
        return json.loads(output)

    def chassis_create(self, params=''):
        """Create baremetal chassis and add cleanup.

        :param String params: Additional args and kwargs
        :return: JSON object of created chassis
        """
        opts = self.get_opts()
        chassis = self.openstack('baremetal chassis create {0} {1}'
                                 .format(opts, params))

        chassis = json.loads(chassis)
        if not chassis:
            self.fail('Baremetal chassis has not been created!')
        self.addCleanup(self.chassis_delete, chassis['uuid'], True)

        return chassis

    def chassis_delete(self, uuid, ignore_exceptions=False):
        """Try to delete baremetal chassis by UUID.

        :param String uuid: UUID of the chassis
        :param Bool ignore_exceptions: Ignore exception (needed for cleanUp)
        :return: raw values output
        :raise: CommandFailed exception when command fails to delete a chassis
        """
        try:
            return self.openstack('baremetal chassis delete {0}'
                                  .format(uuid))
        except exceptions.CommandFailed:
            if not ignore_exceptions:
                raise

    def chassis_list(self, fields=None, params=''):
        """List baremetal chassis.

        :param List fields: List of fields to show
        :param String params: Additional kwargs
        :return: list of JSON chassis objects
        """
        opts = self.get_opts(fields=fields)
        output = self.openstack('baremetal chassis list {0} {1}'
                                .format(opts, params))
        return json.loads(output)

    def chassis_show(self, uuid, fields=None, params=''):
        """Show specified baremetal chassis.

        :param String uuid: UUID of the chassis
        :param List fields: List of fields to show
        :param List params: Additional kwargs
        :return: JSON object of chassis
        """
        opts = self.get_opts(fields)
        chassis = self.openstack('baremetal chassis show {0} {1} {2}'
                                 .format(opts, uuid, params))
        return json.loads(chassis)

    def driver_show(self, driver_name, fields=None, params=''):
        """Show specified baremetal driver.

        :param String driver_name: Name of the driver
        :param List fields: List of fields to show
        :param List params: Additional kwargs
        :return: JSON object of driver
        """
        opts = self.get_opts(fields=fields)
        driver = self.openstack('baremetal driver show {0} {1} {2}'
                                .format(opts, driver_name, params))
        return json.loads(driver)

    def driver_list(self, fields=None, params=''):
        """List baremetal drivers.

        :param List fields: List of fields to show
        :param String params: Additional kwargs
        :return: list of JSON driver objects
        """
        opts = self.get_opts(fields=fields)
        output = self.openstack('baremetal driver list {0} {1}'
                                .format(opts, params))
        return json.loads(output)

    def conductor_show(self, hostname, fields=None, params=''):
        """Show specified baremetal conductors.

        :param String hostname: hostname of the conductor
        :param List fields: List of fields to show
        :param List params: Additional kwargs
        :return: JSON object of driver
        """
        opts = self.get_opts(fields=fields)
        output = self.openstack('baremetal conductor show {0} {1} {2}'
                                .format(opts, hostname, params))
        return json.loads(output)

    def conductor_list(self, fields=None, params=''):
        """List baremetal conductors.

        :param List fields: List of fields to show
        :param String params: Additional kwargs
        :return: list of JSON driver objects
        """
        opts = self.get_opts(fields=fields)
        output = self.openstack('baremetal conductor list {0} {1}'
                                .format(opts, params))
        return json.loads(output)

    def allocation_create(self, resource_class='allocation-test', params=''):
        opts = self.get_opts()
        output = self.openstack('baremetal allocation create {0} '
                                '--resource-class {1} {2}'
                                .format(opts, resource_class, params))
        allocation = json.loads(output)
        self.addCleanup(self.allocation_delete, allocation['uuid'], True)
        if not output:
            self.fail('Baremetal allocation has not been created!')

        return allocation

    def allocation_list(self, fields=None, params=''):
        """List baremetal allocations.

        :param List fields: List of fields to show
        :param String params: Additional kwargs
        :return: list of JSON allocation objects
        """
        opts = self.get_opts(fields=fields)
        output = self.openstack('baremetal allocation list {0} {1}'
                                .format(opts, params))
        return json.loads(output)

    def allocation_show(self, identifier, fields=None, params=''):
        """Show specified baremetal allocation.

        :param String identifier: Name or UUID of the allocation
        :param List fields: List of fields to show
        :param List params: Additional kwargs
        :return: JSON object of allocation
        """
        opts = self.get_opts(fields)
        output = self.openstack('baremetal allocation show {0} {1} {2}'
                                .format(opts, identifier, params))
        return json.loads(output)

    def allocation_delete(self, identifier, ignore_exceptions=False):
        """Try to delete baremetal allocation by name or UUID.

        :param String identifier: Name or UUID of the allocation
        :param Bool ignore_exceptions: Ignore exception (needed for cleanUp)
        :return: raw values output
        :raise: CommandFailed exception when command fails to delete
            an allocation
        """
        try:
            return self.openstack('baremetal allocation delete {0}'
                                  .format(identifier))
        except exceptions.CommandFailed:
            if not ignore_exceptions:
                raise

    def deploy_template_create(self, name, params=''):
        """Create baremetal deploy template and add cleanup.

        :param String name: deploy template name
        :param String params: additional parameters
        :return: JSON object of created deploy template
        """
        opts = self.get_opts()
        template = self.openstack('baremetal deploy template create {0} {1} '
                                  '{2}'.format(opts, name, params))
        template = json.loads(template)
        if not template:
            self.fail('Baremetal deploy template has not been created!')
        self.addCleanup(self.deploy_template_delete, template['uuid'], True)
        return template

    def deploy_template_list(self, fields=None, params=''):
        """List baremetal deploy templates.

        :param List fields: List of fields to show
        :param String params: Additional kwargs
        :return: list of JSON deploy template objects
        """
        opts = self.get_opts(fields=fields)
        output = self.openstack('baremetal deploy template list {0} {1}'
                                .format(opts, params))
        return json.loads(output)

    def deploy_template_show(self, identifier, fields=None, params=''):
        """Show specified baremetal deploy template.

        :param String identifier: Name or UUID of the deploy template
        :param List fields: List of fields to show
        :param List params: Additional kwargs
        :return: JSON object of deploy template
        """
        opts = self.get_opts(fields)
        output = self.openstack('baremetal deploy template show {0} {1} {2}'
                                .format(opts, identifier, params))
        return json.loads(output)

    def deploy_template_delete(self, identifier, ignore_exceptions=False):
        """Try to delete baremetal deploy template by UUID.

        :param String identifier: Name or UUID of the deploy template
        :param Bool ignore_exceptions: Ignore exception (needed for cleanUp)
        :return: raw values output
        :raise: CommandFailed exception when command fails to delete a deploy
            template
        """
        try:
            return self.openstack('baremetal deploy template delete {0}'
                                  .format(identifier))
        except exceptions.CommandFailed:
            if not ignore_exceptions:
                raise



# Copyright 2016 Hitachi, Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ironicclient.common import base
from ironicclient.common.i18n import _
from ironicclient.common import utils
from ironicclient import exc


class VolumeTarget(base.Resource):
    def __repr__(self):
        return "<VolumeTarget %s>" % self._info


class VolumeTargetManager(base.CreateManager):
    resource_class = VolumeTarget
    _creation_attributes = ['extra', 'node_uuid', 'volume_type',
                            'properties', 'boot_index', 'volume_id',
                            'uuid']
    _resource_name = 'volume/targets'

    def list(self, node=None, limit=None, marker=None, sort_key=None,
             sort_dir=None, detail=False, fields=None,
             os_ironic_api_version=None, global_request_id=None):
        """Retrieve a list of volume target.

        :param node:   Optional, UUID or name of a node, to get volume
                       targets for this node only.
        :param marker: Optional, the UUID of a volume target, eg the last
                       volume target from a previous result set. Return
                       the next result set.
        :param limit: The maximum number of results to return per
                      request, if:

            1) limit > 0, the maximum number of volume targets to return.
            2) limit == 0, return the entire list of volume targets.
            3) limit param is NOT specified (None), the number of items
               returned respect the maximum imposed by the Ironic API
               (see Ironic's api.max_limit option).

        :param sort_key: Optional, field used for sorting.

        :param sort_dir: Optional, direction of sorting, either 'asc' (the
                         default) or 'desc'.

        :param detail: Optional, boolean whether to return detailed information
                       about volume targets.

        :param fields: Optional, a list with a specified set of fields
                       of the resource to be returned. Can not be used
                       when 'detail' is set.

        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.

        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :returns: A list of volume targets.

        """
        if limit is not None:
            limit = int(limit)

        if detail and fields:
            raise exc.InvalidAttribute(_("Can't fetch a subset of fields "
                                         "with 'detail' set"))

        filters = utils.common_filters(marker=marker, limit=limit,
                                       sort_key=sort_key, sort_dir=sort_dir,
                                       fields=fields, detail=detail)
        if node is not None:
            filters.append('node=%s' % node)

        path = ''
        if filters:
            path += '?' + '&'.join(filters)
        header_values = {"os_ironic_api_version": os_ironic_api_version,
                         "global_request_id": global_request_id}
        if limit is None:
            return self._list(self._path(path), "targets", **header_values)
        else:
            return self._list_pagination(self._path(path), "targets",
                                         limit=limit, **header_values)

    def get(self, volume_target_id, fields=None, os_ironic_api_version=None,
            global_request_id=None):
        return self._get(resource_id=volume_target_id, fields=fields,
                         os_ironic_api_version=os_ironic_api_version,
                         global_request_id=global_request_id)

    def delete(self, volume_target_id, os_ironic_api_version=None,
               global_request_id=None):
        return self._delete(resource_id=volume_target_id,
                            os_ironic_api_version=os_ironic_api_version,
                            global_request_id=global_request_id)

    def update(self, volume_target_id, patch, os_ironic_api_version=None,
               global_request_id=None):
        return self._update(resource_id=volume_target_id, patch=patch,
                            os_ironic_api_version=os_ironic_api_version,
                            global_request_id=global_request_id)



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ironicclient.tests.functional.osc.v1 import base


class BaremetalDriverTests(base.TestCase):
    """Functional tests for baremetal driver commands."""

    def test_show(self):
        """Show specified driver.

        Test step:
        1) Check output of baremetal driver show command.
        """
        driver = self.driver_show(self.driver_name)
        self.assertEqual(self.driver_name, driver['name'])

    def test_list(self):
        """List available drivers.

        Test steps:
        1) Get list of drivers.
        2) Check that list of drivers is not empty.
        """
        drivers = [
            driver['Supported driver(s)'] for driver in self.driver_list()
        ]
        self.assertIn(self.driver_name, drivers)



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

import ddt
from tempest.lib.common.utils import data_utils

from ironicclient.tests.functional.osc.v1 import base


@ddt.ddt
class BaremetalNodeTests(base.TestCase):
    """Functional tests for baremetal node commands."""

    def setUp(self):
        super(BaremetalNodeTests, self).setUp()
        self.node = self.node_create()

    def test_create_name_uuid(self):
        """Check baremetal node create command with name and UUID.

        Test steps:
        1) Create baremetal node in setUp.
        2) Create one more baremetal node explicitly
        with specified name and UUID.
        3) Check that node successfully created.
        """
        uuid = data_utils.rand_uuid()
        name = data_utils.rand_name('baremetal-node')
        node_info = self.node_create(name=name,
                                     params='--uuid {0}'.format(uuid))
        self.assertEqual(node_info['uuid'], uuid)
        self.assertEqual(node_info['name'], name)
        self.assertEqual(node_info['driver'], self.driver_name)
        self.assertEqual(node_info['maintenance'], False)
        self.assertEqual(node_info['provision_state'], 'enroll')
        node_list = self.node_list()
        self.assertIn(uuid, [x['UUID'] for x in node_list])
        self.assertIn(name, [x['Name'] for x in node_list])

    def test_create_old_api_version(self):
        """Check baremetal node create command with name and UUID.

        Test steps:
        1) Create baremetal node in setUp.
        2) Create one more baremetal node explicitly with old API version
        3) Check that node successfully created.
        """
        node_info = self.node_create(
            params='--os-baremetal-api-version 1.5')
        self.assertEqual(node_info['driver'], self.driver_name)
        self.assertEqual(node_info['maintenance'], False)
        self.assertEqual(node_info['provision_state'], 'available')

    @ddt.data('name', 'uuid')
    def test_delete(self, key):
        """Check baremetal node delete command with name/UUID argument.

        Test steps:
        1) Create baremetal node in setUp.
        2) Delete baremetal node by name/UUID.
        3) Check that node deleted successfully.
        """
        output = self.node_delete(self.node[key])
        self.assertIn('Deleted node {0}'.format(self.node[key]), output)
        node_list = self.node_list()
        self.assertNotIn(self.node['name'], [x['Name'] for x in node_list])
        self.assertNotIn(self.node['uuid'], [x['UUID'] for x in node_list])

    def test_list(self):
        """Check baremetal node list command.

        Test steps:
        1) Create baremetal node in setUp.
        2) List baremetal nodes.
        3) Check node name in nodes list.
        """
        node_list = self.node_list()
        self.assertIn(self.node['name'], [x['Name'] for x in node_list])
        self.assertIn(self.node['uuid'], [x['UUID'] for x in node_list])

    @ddt.data('name', 'uuid')
    def test_set(self, key):
        """Check baremetal node set command calling it by name/UUID.

        Test steps:
        1) Create baremetal node in setUp.
        2) Set another name for node calling it by name/UUID.
        3) Check that baremetal node name was changed.
        """
        new_name = data_utils.rand_name('newnodename')
        self.openstack('baremetal node set --name {0} {1}'
                       .format(new_name, self.node[key]))
        show_prop = self.node_show(self.node['uuid'], ['name'])
        self.assertEqual(new_name, show_prop['name'])

    @ddt.data('name', 'uuid')
    def test_unset(self, key):
        """Check baremetal node unset command calling it by node name/UUID.

        Test steps:
        1) Create baremetal node in setUp.
        2) Unset name of baremetal node calling it by node name/UUID.
        3) Check that node has no more name.
        """
        self.openstack('baremetal node unset --name {0}'
                       .format(self.node[key]))
        show_prop = self.node_show(self.node['uuid'], ['name'])
        self.assertIsNone(show_prop['name'])

    @ddt.data('name', 'uuid')
    def test_show(self, key):
        """Check baremetal node show command with name and UUID arguments.

        Test steps:
        1) Create baremetal node in setUp.
        2) Show baremetal node calling it with name and UUID arguments.
        3) Check name, uuid and driver in node show output.
        """
        node = self.node_show(self.node[key],
                              ['name', 'uuid', 'driver'])
        self.assertEqual(self.node['name'], node['name'])
        self.assertEqual(self.node['uuid'], node['uuid'])
        self.assertEqual(self.node['driver'], node['driver'])

    def test_baremetal_node_maintenance_set_unset(self):
        """Check baremetal node maintenance set command.

        Test steps:
        1) Create baremetal node in setUp.
        2) Check maintenance status of fresh node is False.
        3) Set maintenance status for node.
        4) Check maintenance status of node is True.
        5) Unset maintenance status for node.
        6) Check maintenance status of node is False back.
        """
        show_prop = self.node_show(self.node['name'], ['maintenance'])
        self.assertFalse(show_prop['maintenance'])

        self.openstack('baremetal node maintenance set {0}'.
                       format(self.node['name']))

        show_prop = self.node_show(self.node['name'], ['maintenance'])
        self.assertTrue(show_prop['maintenance'])

        self.openstack('baremetal node maintenance unset {0}'.
                       format(self.node['name']))

        show_prop = self.node_show(self.node['name'], ['maintenance'])
        self.assertFalse(show_prop['maintenance'])

    def test_baremetal_node_maintenance_set_unset_reason(self):
        """Check baremetal node maintenance set command.

        Test steps:
        1) Create baremetal node in setUp.
        2) Check initial maintenance reason is None.
        3) Set maintenance status for node with reason.
        4) Check maintenance reason of node equals to expected value.
           Also check maintenance status.
        5) Unset maintenance status for node. Recheck maintenance status.
        6) Check maintenance reason is None. Recheck maintenance status.
        """
        reason = "Hardware maintenance."
        show_prop = self.node_show(self.node['name'],
                                   ['maintenance_reason', 'maintenance'])
        self.assertIsNone(show_prop['maintenance_reason'])
        self.assertFalse(show_prop['maintenance'])

        self.openstack("baremetal node maintenance set --reason '{0}' {1}".
                       format(reason, self.node['name']))

        show_prop = self.node_show(self.node['name'],
                                   ['maintenance_reason', 'maintenance'])
        self.assertEqual(reason, show_prop['maintenance_reason'])
        self.assertTrue(show_prop['maintenance'])

        self.openstack('baremetal node maintenance unset {0}'.
                       format(self.node['name']))

        show_prop = self.node_show(self.node['name'],
                                   ['maintenance_reason', 'maintenance'])
        self.assertIsNone(show_prop['maintenance_reason'])
        self.assertFalse(show_prop['maintenance'])

    @ddt.data(
        (50, '1'),
        ('MAX', 'JBOD'),
        (300, '6+0')
    )
    @ddt.unpack
    def test_set_unset_target_raid_config(self, size, raid_level):
        """Set and unset node target RAID config data.

        Test steps:
        1) Create baremetal node in setUp.
        2) Set target RAID config data for the node
        3) Check target_raid_config of node equals to expected value.
        4) Unset target_raid_config data.
        5) Check that target_raid_config data is empty.
        """
        min_version = '--os-baremetal-api-version 1.12'
        argument_json = {"logical_disks":
                         [{"size_gb": size, "raid_level": raid_level}]}
        argument_string = json.dumps(argument_json)
        self.openstack("baremetal node set --target-raid-config '{}' {} {}"
                       .format(argument_string, self.node['uuid'],
                               min_version))

        show_prop = self.node_show(self.node['uuid'], ['target_raid_config'],
                                   min_version)
        self.assert_dict_is_subset(argument_json,
                                   show_prop['target_raid_config'])

        self.openstack("baremetal node unset --target-raid-config {} {}"
                       .format(self.node['uuid'], min_version))
        show_prop = self.node_show(self.node['uuid'], ['target_raid_config'],
                                   min_version)
        self.assertEqual({}, show_prop['target_raid_config'])



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import logging

from osc_lib.command import command

from ironicclient.common.i18n import _
from ironicclient.v1 import create_resources


class CreateBaremetal(command.Command):
    """Create resources from files"""

    log = logging.getLogger(__name__ + ".CreateBaremetal")

    def get_parser(self, prog_name):
        parser = super(CreateBaremetal, self).get_parser(prog_name)

        parser.add_argument(
            "resource_files", metavar="<file>", nargs="+",
            help=_("File (.yaml or .json) containing descriptions of the "
                   "resources to create. Can be specified multiple times."))
        return parser

    def take_action(self, parsed_args):
        create_resources.create_resources(self.app.client_manager.baremetal,
                                          parsed_args.resource_files)



# -*- coding: utf-8 -*-
#
# Copyright © 2013 Red Hat, Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ironicclient.common import base
from ironicclient.common.i18n import _
from ironicclient.common import utils
from ironicclient import exc


class Chassis(base.Resource):
    def __repr__(self):
        return "<Chassis %s>" % self._info


class ChassisManager(base.CreateManager):
    resource_class = Chassis
    _resource_name = 'chassis'
    _creation_attributes = ['description', 'extra', 'uuid']

    def list(self, marker=None, limit=None, sort_key=None,
             sort_dir=None, detail=False, fields=None,
             os_ironic_api_version=None, global_request_id=None):
        """Retrieve a list of chassis.

        :param marker: Optional, the UUID of a chassis, eg the last
                       chassis from a previous result set. Return
                       the next result set.
        :param limit: The maximum number of results to return per
                      request, if:

            1) limit > 0, the maximum number of chassis to return.
            2) limit == 0, return the entire list of chassis.
            3) limit param is NOT specified (None), the number of items
               returned respect the maximum imposed by the Ironic API
               (see Ironic's api.max_limit option).

        :param sort_key: Optional, field used for sorting.

        :param sort_dir: Optional, direction of sorting, either 'asc' (the
                         default) or 'desc'.

        :param detail: Optional, boolean whether to return detailed information
                       about chassis.

        :param fields: Optional, a list with a specified set of fields
                       of the resource to be returned. Can not be used
                       when 'detail' is set.

        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.

        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :returns: A list of chassis.

        """
        if limit is not None:
            limit = int(limit)

        if detail and fields:
            raise exc.InvalidAttribute(_("Can't fetch a subset of fields "
                                         "with 'detail' set"))

        filters = utils.common_filters(marker, limit, sort_key, sort_dir,
                                       fields)

        path = ''
        if detail:
            path += 'detail'
        if filters:
            path += '?' + '&'.join(filters)
        header_values = {"os_ironic_api_version": os_ironic_api_version,
                         "global_request_id": global_request_id}
        if limit is None:
            return self._list(self._path(path), "chassis", **header_values)
        else:
            return self._list_pagination(self._path(path), "chassis",
                                         limit=limit, **header_values)

    def list_nodes(self, chassis_id, marker=None, limit=None,
                   sort_key=None, sort_dir=None, detail=False, fields=None,
                   associated=None, maintenance=None, provision_state=None,
                   os_ironic_api_version=None, global_request_id=None):
        """List all the nodes for a given chassis.

        :param chassis_id: The UUID of the chassis.
        :param marker: Optional, the UUID of a node, eg the last
                       node from a previous result set. Return
                       the next result set.
        :param limit: The maximum number of results to return per
                      request, if:

            1) limit > 0, the maximum number of nodes to return.
            2) limit == 0, return the entire list of nodes.
            3) limit param is NOT specified (None), the number of items
               returned respect the maximum imposed by the Ironic API
               (see Ironic's api.max_limit option).

        :param sort_key: Optional, field used for sorting.

        :param sort_dir: Optional, direction of sorting, either 'asc' (the
                         default) or 'desc'.

        :param detail: Optional, boolean whether to return detailed information
                       about nodes.

        :param fields: Optional, a list with a specified set of fields
                       of the resource to be returned. Can not be used
                       when 'detail' is set.

        :param associated: Optional. Either a Boolean or a string
                           representation of a Boolean that indicates whether
                           to return a list of associated (True or "True") or
                           unassociated (False or "False") nodes.

        :param maintenance: Optional. Either a Boolean or a string
                            representation of a Boolean that indicates whether
                            to return nodes in maintenance mode (True or
                            "True"), or not in maintenance mode (False or
                            "False").

        :param provision_state: Optional. String value to get only nodes in
                                that provision state.

        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.

        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :returns: A list of nodes.

        """
        if limit is not None:
            limit = int(limit)

        if detail and fields:
            raise exc.InvalidAttribute(_("Can't fetch a subset of fields "
                                         "with 'detail' set"))

        filters = utils.common_filters(marker, limit, sort_key, sort_dir,
                                       fields)

        if associated is not None:
            filters.append('associated=%s' % associated)
        if maintenance is not None:
            filters.append('maintenance=%s' % maintenance)
        if provision_state is not None:
            filters.append('provision_state=%s' % provision_state)

        path = "%s/nodes" % chassis_id
        if detail:
            path += '/detail'

        if filters:
            path += '?' + '&'.join(filters)
        header_values = {"os_ironic_api_version": os_ironic_api_version,
                         "global_request_id": global_request_id}
        if limit is None:
            return self._list(self._path(path), "nodes", **header_values)
        else:
            return self._list_pagination(self._path(path), "nodes",
                                         limit=limit, **header_values)

    def get(self, chassis_id, fields=None, os_ironic_api_version=None,
            global_request_id=None):
        return self._get(resource_id=chassis_id, fields=fields,
                         os_ironic_api_version=os_ironic_api_version,
                         global_request_id=global_request_id)

    def delete(self, chassis_id, os_ironic_api_version=None,
               global_request_id=None):
        return self._delete(resource_id=chassis_id,
                            os_ironic_api_version=os_ironic_api_version,
                            global_request_id=global_request_id)

    def update(self, chassis_id, patch, os_ironic_api_version=None,
               global_request_id=None):
        return self._update(resource_id=chassis_id, patch=patch,
                            os_ironic_api_version=os_ironic_api_version,
                            global_request_id=global_request_id)



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os

from oslo_utils import strutils

from ironicclient.common import base
from ironicclient.common.i18n import _
from ironicclient.common import utils
from ironicclient import exc
from ironicclient.v1 import volume_connector
from ironicclient.v1 import volume_target

_power_states = {
    'on': 'power on',
    'off': 'power off',
    'reboot': 'rebooting',
    'soft off': 'soft power off',
    'soft reboot': 'soft rebooting',
}


LOG = logging.getLogger(__name__)
_DEFAULT_POLL_INTERVAL = 2


class Node(base.Resource):
    def __repr__(self):
        return "<Node %s>" % self._info


class NodeManager(base.CreateManager):
    resource_class = Node
    _creation_attributes = ['chassis_uuid', 'driver', 'driver_info',
                            'extra', 'uuid', 'properties', 'name',
                            'bios_interface', 'boot_interface',
                            'console_interface', 'deploy_interface',
                            'inspect_interface', 'management_interface',
                            'network_interface', 'power_interface',
                            'raid_interface', 'rescue_interface',
                            'storage_interface', 'vendor_interface',
                            'resource_class', 'conductor_group',
                            'automated_clean', 'network_data']
    _resource_name = 'nodes'

    def list(self, associated=None, maintenance=None, marker=None,
             limit=None, detail=False, sort_key=None, sort_dir=None,
             fields=None, provision_state=None, driver=None,
             resource_class=None, chassis=None, fault=None,
             os_ironic_api_version=None, conductor_group=None,
             conductor=None, owner=None, retired=None, lessee=None,
             global_request_id=None):
        """Retrieve a list of nodes.

        :param associated: Optional. Either a Boolean or a string
                           representation of a Boolean that indicates whether
                           to return a list of associated (True or "True") or
                           unassociated (False or "False") nodes.
        :param maintenance: Optional. Either a Boolean or a string
                            representation of a Boolean that indicates whether
                            to return nodes in maintenance mode (True or
                            "True"), or not in maintenance mode (False or
                            "False").
        :param retired: Optional. Either a Boolean or a string representation
                        of a Boolean that indicates whether to return retired
                        nodes (True or "True").
        :param provision_state: Optional. String value to get only nodes in
                                that provision state.
        :param marker: Optional, the UUID of a node, eg the last
                       node from a previous result set. Return
                       the next result set.
        :param limit: The maximum number of results to return per
                      request, if:

            1) limit > 0, the maximum number of nodes to return.
            2) limit == 0, return the entire list of nodes.
            3) limit param is NOT specified (None), the number of items
               returned respect the maximum imposed by the Ironic API
               (see Ironic's api.max_limit option).

        :param detail: Optional, boolean whether to return detailed information
                       about nodes.

        :param sort_key: Optional, field used for sorting.

        :param sort_dir: Optional, direction of sorting, either 'asc' (the
                         default) or 'desc'.

        :param fields: Optional, a list with a specified set of fields
                       of the resource to be returned. Can not be used
                       when 'detail' is set.

        :param driver: Optional. String value to get only nodes using that
                       driver.

        :param resource_class: Optional. String value to get only nodes
                               with the given resource class set.

        :param chassis: Optional, the UUID of a chassis. Used to get only
                        nodes of this chassis.

        :param fault: Optional. String value to get only nodes with
                      specified fault.

        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.

        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :param conductor_group: Optional. String value to get only nodes
                                with the given conductor group set.
        :param conductor: Optional. String value to get only nodes
                          mapped to the given conductor.
        :param owner: Optional. String value to get only nodes
                          mapped to a specific owner.
        :param lessee: Optional. String value to get only nodes
                          mapped to a specific lessee.

        :returns: A list of nodes.

        """
        if limit is not None:
            limit = int(limit)

        if detail and fields:
            raise exc.InvalidAttribute(_("Can't fetch a subset of fields "
                                         "with 'detail' set"))

        filters = utils.common_filters(marker, limit, sort_key, sort_dir,
                                       fields)
        if associated is not None:
            filters.append('associated=%s' % associated)
        if maintenance is not None:
            filters.append('maintenance=%s' % maintenance)
        if retired is not None:
            filters.append('retired=%s' % retired)
        if fault is not None:
            filters.append('fault=%s' % fault)
        if provision_state is not None:
            filters.append('provision_state=%s' % provision_state)
        if driver is not None:
            filters.append('driver=%s' % driver)
        if resource_class is not None:
            filters.append('resource_class=%s' % resource_class)
        if chassis is not None:
            filters.append('chassis_uuid=%s' % chassis)
        if conductor_group is not None:
            filters.append('conductor_group=%s' % conductor_group)
        if conductor is not None:
            filters.append('conductor=%s' % conductor)
        if owner is not None:
            filters.append('owner=%s' % owner)
        if lessee is not None:
            filters.append('lessee=%s' % lessee)

        path = ''
        if detail:
            path += 'detail'
        if filters:
            path += '?' + '&'.join(filters)
        header_values = {"os_ironic_api_version": os_ironic_api_version,
                         "global_request_id": global_request_id}
        if limit is None:
            return self._list(self._path(path), "nodes", **header_values)
        else:
            return self._list_pagination(
                self._path(path), "nodes", limit=limit, **header_values)

    def list_ports(self, node_id, marker=None, limit=None, sort_key=None,
                   sort_dir=None, detail=False, fields=None,
                   os_ironic_api_version=None, global_request_id=None):
        """List all the ports for a given node.

        :param node_id: Name or UUID of the node.
        :param marker: Optional, the UUID of a port, eg the last
                       port from a previous result set. Return
                       the next result set.
        :param limit: The maximum number of results to return per
                      request, if:

            1) limit > 0, the maximum number of ports to return.
            2) limit == 0, return the entire list of ports.
            3) limit param is NOT specified (None), the number of items
               returned respect the maximum imposed by the Ironic API
               (see Ironic's api.max_limit option).

        :param sort_key: Optional, field used for sorting.

        :param sort_dir: Optional, direction of sorting, either 'asc' (the
                         default) or 'desc'.

        :param detail: Optional, boolean whether to return detailed information
                       about ports.

        :param fields: Optional, a list with a specified set of fields
                       of the resource to be returned. Can not be used
                       when 'detail' is set.

        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.

        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :returns: A list of ports.

        """
        if limit is not None:
            limit = int(limit)

        if detail and fields:
            raise exc.InvalidAttribute(_("Can't fetch a subset of fields "
                                         "with 'detail' set"))

        filters = utils.common_filters(marker, limit, sort_key, sort_dir,
                                       fields)

        path = "%s/ports" % node_id
        if detail:
            path += '/detail'

        if filters:
            path += '?' + '&'.join(filters)
        header_values = {"os_ironic_api_version": os_ironic_api_version,
                         "global_request_id": global_request_id}
        if limit is None:
            return self._list(self._path(path), "ports", **header_values)
        else:
            return self._list_pagination(
                self._path(path), "ports", limit=limit, **header_values)

    def list_volume_connectors(self, node_id, marker=None, limit=None,
                               sort_key=None, sort_dir=None, detail=False,
                               fields=None, os_ironic_api_version=None,
                               global_request_id=None):
        """List all the volume connectors for a given node.

        :param node_id: Name or UUID of the node.
        :param marker: Optional, the UUID of a volume connector, eg the last
                       volume connector from a previous result set. Return
                       the next result set.
        :param limit: The maximum number of results to return per
                      request, if:

            1) limit > 0, the maximum number of volume connectors to return.
            2) limit == 0, return the entire list of volume connectors.
            3) limit param is NOT specified (None), the number of items
               returned respect the maximum imposed by the Ironic API
               (see Ironic's api.max_limit option).

        :param sort_key: Optional, field used for sorting.

        :param sort_dir: Optional, direction of sorting, either 'asc' (the
                         default) or 'desc'.

        :param detail: Optional, boolean whether to return detailed information
                       about volume connectors.

        :param fields: Optional, a list with a specified set of fields
                       of the resource to be returned. Can not be used
                       when 'detail' is set.

        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.

        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :returns: A list of volume connectors.

        """
        if limit is not None:
            limit = int(limit)

        if detail and fields:
            raise exc.InvalidAttribute(_("Can't fetch a subset of fields "
                                         "with 'detail' set"))

        filters = utils.common_filters(marker=marker, limit=limit,
                                       sort_key=sort_key, sort_dir=sort_dir,
                                       fields=fields, detail=detail)

        path = "%s/volume/connectors" % node_id
        if filters:
            path += '?' + '&'.join(filters)
        header_values = {"os_ironic_api_version": os_ironic_api_version,
                         "global_request_id": global_request_id}
        if limit is None:
            return self._list(self._path(path), response_key="connectors",
                              obj_class=volume_connector.VolumeConnector,
                              **header_values)
        else:
            return self._list_pagination(
                self._path(path), response_key="connectors", limit=limit,
                obj_class=volume_connector.VolumeConnector, **header_values)

    def list_volume_targets(self, node_id, marker=None, limit=None,
                            sort_key=None, sort_dir=None, detail=False,
                            fields=None, os_ironic_api_version=None,
                            global_request_id=None):
        """List all the volume targets for a given node.

        :param node_id: Name or UUID of the node.
        :param marker: Optional, the UUID of a volume target, eg the last
                       volume target from a previous result set. Return
                       the next result set.
        :param limit: The maximum number of results to return per
                      request, if:

            1) limit > 0, the maximum number of volume targets to return.
            2) limit == 0, return the entire list of volume targets.
            3) limit param is NOT specified (None), the number of items
               returned respect the maximum imposed by the Ironic API
               (see Ironic's api.max_limit option).

        :param sort_key: Optional, field used for sorting.

        :param sort_dir: Optional, direction of sorting, either 'asc' (the
                         default) or 'desc'.

        :param detail: Optional, boolean whether to return detailed information
                       about volume targets.

        :param fields: Optional, a list with a specified set of fields
                       of the resource to be returned. Can not be used
                       when 'detail' is set.

        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.

        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :returns: A list of volume targets.

        """
        if limit is not None:
            limit = int(limit)

        if detail and fields:
            raise exc.InvalidAttribute(_("Can't fetch a subset of fields "
                                         "with 'detail' set"))

        filters = utils.common_filters(marker=marker, limit=limit,
                                       sort_key=sort_key, sort_dir=sort_dir,
                                       fields=fields, detail=detail)

        path = "%s/volume/targets" % node_id
        if filters:
            path += '?' + '&'.join(filters)
        header_values = {"os_ironic_api_version": os_ironic_api_version,
                         "global_request_id": global_request_id}
        if limit is None:
            return self._list(self._path(path), response_key="targets",
                              obj_class=volume_target.VolumeTarget,
                              **header_values)
        else:
            return self._list_pagination(
                self._path(path), response_key="targets", limit=limit,
                obj_class=volume_target.VolumeTarget, **header_values)

    def get(self, node_id, fields=None, os_ironic_api_version=None,
            global_request_id=None):
        return self._get(resource_id=node_id, fields=fields,
                         os_ironic_api_version=os_ironic_api_version,
                         global_request_id=global_request_id)

    def get_by_instance_uuid(
            self, instance_uuid, fields=None,
            os_ironic_api_version=None, global_request_id=None):
        path = '?instance_uuid=%s' % instance_uuid
        if fields is not None:
            path += '&fields=' + ','.join(fields)
        else:
            path = 'detail' + path

        nodes = self._list(
            self._path(path), 'nodes',
            os_ironic_api_version=os_ironic_api_version,
            global_request_id=global_request_id)
        # get all the details of the node assuming that
        # filtering by instance_uuid returns a collection
        # of one node if successful.
        if len(nodes) == 1:
            return nodes[0]
        else:
            raise exc.NotFound()

    def delete(self, node_id, os_ironic_api_version=None,
               global_request_id=None):
        return self._delete(
            resource_id=node_id,
            os_ironic_api_version=os_ironic_api_version,
            global_request_id=global_request_id)

    def update(self, node_id, patch, http_method='PATCH',
               os_ironic_api_version=None, reset_interfaces=None,
               global_request_id=None):
        params = {}
        if reset_interfaces is not None:
            params['reset_interfaces'] = reset_interfaces
        return self._update(resource_id=node_id, patch=patch,
                            method=http_method,
                            os_ironic_api_version=os_ironic_api_version,
                            global_request_id=global_request_id,
                            params=params)

    def vendor_passthru(self, node_id, method, args=None,
                        http_method=None, os_ironic_api_version=None,
                        global_request_id=None):
        """Issue requests for vendor-specific actions on a given node.

        :param node_id: The UUID of the node.
        :param method: Name of the vendor method.
        :param args: Optional. The arguments to be passed to the method.
        :param http_method: The HTTP method to use on the request.
                            Defaults to POST.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        """
        if args is None:
            args = {}

        if http_method is None:
            http_method = 'POST'

        http_method = http_method.upper()
        header_values = {"os_ironic_api_version": os_ironic_api_version,
                         "global_request_id": global_request_id}

        path = "%s/vendor_passthru/%s" % (node_id, method)
        if http_method in ('POST', 'PUT', 'PATCH'):
            return self.update(path, args, http_method=http_method,
                               **header_values)
        elif http_method == 'DELETE':
            return self.delete(path, **header_values)
        elif http_method == 'GET':
            return self.get(path, **header_values)
        else:
            raise exc.InvalidAttribute(
                _('Unknown HTTP method: %s') % http_method)

    def vif_list(self, node_ident, os_ironic_api_version=None,
                 global_request_id=None):
        """List VIFs attached to a given node.

        :param node_ident: The UUID or Name of the node.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        """
        path = "%s/vifs" % node_ident

        return self._list(self._path(path), "vifs",
                          os_ironic_api_version=os_ironic_api_version,
                          global_request_id=global_request_id)

    def vif_attach(self, node_ident, vif_id, os_ironic_api_version=None,
                   global_request_id=None, **kwargs):
        """Attach VIF to a given node.

        :param node_ident: The UUID or Name of the node.
        :param vif_id: The UUID or Name of the VIF to attach.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        :param kwargs: A dictionary containing the attributes of the resource
                       that will be created.
        """
        path = "%s/vifs" % node_ident
        data = {"id": vif_id}
        if 'id' in kwargs:
            raise exc.InvalidAttribute("The attribute 'id' can't be "
                                       "specified in vif-info")
        data.update(kwargs)
        # TODO(vdrok): cleanup places doing custom path and http_method
        self.update(path, data, http_method="POST",
                    os_ironic_api_version=os_ironic_api_version,
                    global_request_id=global_request_id)

    def vif_detach(self, node_ident, vif_id, os_ironic_api_version=None,
                   global_request_id=None):
        """Detach VIF from a given node.

        :param node_ident: The UUID or Name of the node.
        :param vif_id: The UUID or Name of the VIF to detach.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        """
        path = "%s/vifs/%s" % (node_ident, vif_id)
        self.delete(path, os_ironic_api_version=os_ironic_api_version,
                    global_request_id=global_request_id)

    def set_maintenance(self, node_id, state, maint_reason=None,
                        os_ironic_api_version=None, global_request_id=None):
        """Set the maintenance mode for the node.

        :param node_id: The UUID of the node.
        :param state: the maintenance mode; either a Boolean or a string
                      representation of a Boolean (eg, 'true', 'on', 'false',
                      'off'). True to put the node in maintenance mode; False
                      to take the node out of maintenance mode.
        :param maint_reason: Optional string. Reason for putting node
                             into maintenance mode.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :raises: InvalidAttribute if state is an invalid string (that doesn't
                 represent a Boolean).
        """
        if isinstance(state, bool):
            maintenance_mode = state
        else:
            try:
                maintenance_mode = strutils.bool_from_string(state, True)
            except ValueError as e:
                raise exc.InvalidAttribute(_("Argument 'state': %(err)s") %
                                           {'err': e})
        path = "%s/maintenance" % node_id
        header_values = {"os_ironic_api_version": os_ironic_api_version,
                         "global_request_id": global_request_id}
        if maintenance_mode:
            reason = {'reason': maint_reason}
            return self.update(path, reason, http_method='PUT',
                               **header_values)
        else:
            return self.delete(path, **header_values)

    def set_power_state(self, node_id, state, soft=False, timeout=None,
                        os_ironic_api_version=None, global_request_id=None):
        """Sets power state for a node.

        :param node_id: Node identifier
        :param state: One of target power state, 'on', 'off', or 'reboot'
        :param soft: The flag for graceful power 'off' or 'reboot'
        :param timeout: The timeout (in seconds) positive integer value (> 0)
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :raises: ValueError if 'soft' or 'timeout' option is invalid
        :returns: The status of the request
        """
        if state == 'on' and soft:
            raise ValueError(
                _("'soft' option is invalid for the power-state 'on'"))

        path = "%s/states/power" % node_id

        requested_state = 'soft ' + state if soft else state
        target = _power_states.get(requested_state, state)

        body = {'target': target}
        if timeout is not None:
            msg = _("'timeout' option for setting power state must have "
                    "positive integer value (> 0)")
            try:
                timeout = int(timeout)
            except (ValueError, TypeError):
                raise ValueError(msg)

            if timeout <= 0:
                raise ValueError(msg)
            body = {'target': target, 'timeout': timeout}

        return self.update(path, body, http_method='PUT',
                           os_ironic_api_version=os_ironic_api_version,
                           global_request_id=global_request_id)

    def set_boot_mode(self, node_id, state,
                      os_ironic_api_version=None, global_request_id=None):
        """Sets boot mode for a node.

        :param node_id: Node identifier
        :param state: One of target boot modes, 'uefi' or 'bios'
        :param os_ironic_api_version: String version (e.g. "1.76") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :raises: ValueError if boot mode is not one of 'uefi' / 'bios'
        :returns: The status of the request
        """
        if state not in ('uefi', 'bios'):
            raise ValueError(
                _("Valid boot modes are 'uefi' or 'bios'"))

        path = "%s/states/boot_mode" % node_id
        target = state
        body = {'target': target}

        return self.update(path, body, http_method='PUT',
                           os_ironic_api_version=os_ironic_api_version,
                           global_request_id=global_request_id)

    def set_secure_boot(self, node_id, state,
                        os_ironic_api_version=None, global_request_id=None):
        """Set the secure boot state for the node.

        :param node_id: The UUID of the node.
        :param state: the secure boot state; either a Boolean or a string
                      representation of a Boolean (eg, 'true', 'on', 'false',
                      'off'). True to turn secure boot on; False
                      to turn secure boot off.
        :param os_ironic_api_version: String version (e.g. "1.76") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :raises: InvalidAttribute if state is an invalid string (that doesn't
                 represent a Boolean).
        """
        if isinstance(state, bool):
            target = state
        else:
            try:
                target = strutils.bool_from_string(state, strict=True)
            except ValueError as e:
                raise exc.InvalidAttribute(_("Argument 'state': %(err)s") %
                                           {'err': e})
        path = "%s/states/secure_boot" % node_id
        body = {'target': target}

        return self.update(path, body, http_method='PUT',
                           os_ironic_api_version=os_ironic_api_version,
                           global_request_id=global_request_id)

    def set_target_raid_config(
            self, node_ident, target_raid_config,
            os_ironic_api_version=None, global_request_id=None):
        """Sets target_raid_config for a node.

        :param node_ident: Node identifier
        :param target_raid_config: A dictionary with the target RAID
            configuration; may be empty.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :returns: status of the request
        """
        path = "%s/states/raid" % node_ident
        return self.update(path, target_raid_config, http_method='PUT',
                           os_ironic_api_version=os_ironic_api_version,
                           global_request_id=global_request_id)

    def validate(self, node_uuid, os_ironic_api_version=None,
                 global_request_id=None):
        path = "%s/validate" % node_uuid
        return self.get(path, os_ironic_api_version=os_ironic_api_version,
                        global_request_id=global_request_id)

    def set_provision_state(
            self, node_uuid, state, configdrive=None, cleansteps=None,
            rescue_password=None, os_ironic_api_version=None,
            global_request_id=None, deploysteps=None):
        """Set the provision state for the node.

        :param node_uuid: The UUID or name of the node.
        :param state: The desired provision state. One of 'active', 'deleted',
             'rebuild', 'inspect', 'provide', 'manage', 'clean', 'abort',
             'rescue', 'unrescue'.
        :param configdrive: A gzipped, base64-encoded configuration drive
            string OR the path to the configuration drive file OR the path to
            a directory containing the config drive files OR a dictionary to
            build config drive from. In case it's a directory, a config drive
            will be generated from it. In case it's a dictionary, a config
            drive will be generated on the server side (requires API version
            1.56). This is only valid when setting state to 'active'.
        :param cleansteps: The clean steps as a list of clean-step
            dictionaries; each dictionary should have keys 'interface' and
            'step', and optional key 'args'. This must be specified (and is
            only valid) when setting provision-state to 'clean'.
        :param rescue_password: A string to be used as the login password
            inside the rescue ramdisk once a node is rescued. This must be
            specified (and is only valid) when setting 'state' to 'rescue'.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        :param deploysteps: The deploy steps as a list of deploy-step
            dictionaries; each dictionary should have keys 'interface', 'step',
            'priority', and optional key 'args'. This is optional and is
            only valid when setting provision-state to 'active' or 'rebuild'.
        :raises: InvalidAttribute if there was an error with the clean steps or
            deploy steps
        :returns: The status of the request
        """

        path = "%s/states/provision" % node_uuid
        body = {'target': state}
        if configdrive:
            if not isinstance(configdrive, dict):
                if os.path.isfile(configdrive):
                    with open(configdrive, 'rb') as f:
                        configdrive = f.read()
                if os.path.isdir(configdrive):
                    configdrive = utils.make_configdrive(configdrive)

            if isinstance(configdrive, bytes):
                try:
                    configdrive = configdrive.decode('utf-8')
                except UnicodeError:
                    raise ValueError('Config drive must be a dictionary or '
                                     'a base64 encoded string')
            body['configdrive'] = configdrive
        elif cleansteps:
            body['clean_steps'] = cleansteps
        elif rescue_password:
            body['rescue_password'] = rescue_password

        if deploysteps:
            body['deploy_steps'] = deploysteps

        return self.update(path, body, http_method='PUT',
                           os_ironic_api_version=os_ironic_api_version,
                           global_request_id=global_request_id)

    def states(self, node_uuid, os_ironic_api_version=None,
               global_request_id=None):
        path = "%s/states" % node_uuid
        return self.get(path, os_ironic_api_version=os_ironic_api_version,
                        global_request_id=global_request_id)

    def get_console(self, node_uuid, os_ironic_api_version=None,
                    global_request_id=None):
        path = "%s/states/console" % node_uuid
        return self._get_as_dict(
            path, os_ironic_api_version=os_ironic_api_version,
            global_request_id=global_request_id)

    def set_console_mode(self, node_uuid, enabled, os_ironic_api_version=None,
                         global_request_id=None):
        """Set the console mode for the node.

        :param node_uuid: The UUID of the node.
        :param enabled: Either a Boolean or a string representation of a
                        Boolean. True to enable the console; False to disable.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        """
        path = "%s/states/console" % node_uuid
        target = {'enabled': enabled}
        return self.update(path, target, http_method='PUT',
                           os_ironic_api_version=os_ironic_api_version,
                           global_request_id=global_request_id)

    def set_boot_device(self, node_uuid, boot_device, persistent=False,
                        os_ironic_api_version=None, global_request_id=None):
        path = "%s/management/boot_device" % node_uuid
        target = {'boot_device': boot_device, 'persistent': persistent}
        return self.update(path, target, http_method='PUT',
                           os_ironic_api_version=os_ironic_api_version,
                           global_request_id=global_request_id)

    def get_boot_device(self, node_uuid, os_ironic_api_version=None,
                        global_request_id=None):
        path = "%s/management/boot_device" % node_uuid
        return self._get_as_dict(
            path, os_ironic_api_version=os_ironic_api_version,
            global_request_id=global_request_id)

    def inject_nmi(self, node_uuid, os_ironic_api_version=None,
                   global_request_id=None):
        path = "%s/management/inject_nmi" % node_uuid
        return self.update(path, {}, http_method='PUT',
                           os_ironic_api_version=os_ironic_api_version,
                           global_request_id=global_request_id)

    def get_supported_boot_devices(self, node_uuid, os_ironic_api_version=None,
                                   global_request_id=None):
        path = "%s/management/boot_device/supported" % node_uuid
        return self._get_as_dict(
            path, os_ironic_api_version=os_ironic_api_version,
            global_request_id=global_request_id)

    def get_vendor_passthru_methods(
            self, node_ident, os_ironic_api_version=None,
            global_request_id=None):
        path = "%s/vendor_passthru/methods" % node_ident
        return self._get_as_dict(
            path, os_ironic_api_version=os_ironic_api_version,
            global_request_id=global_request_id)

    def get_traits(self, node_ident, os_ironic_api_version=None,
                   global_request_id=None):
        """Get traits for a node.

        :param node_ident: node UUID or name.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        """
        path = "%s/traits" % node_ident
        return self._list_primitives(
            self._path(path), 'traits',
            os_ironic_api_version=os_ironic_api_version,
            global_request_id=global_request_id)

    def add_trait(self, node_ident, trait, os_ironic_api_version=None,
                  global_request_id=None):
        """Add a trait to a node.

        :param node_ident: node UUID or name.
        :param trait: trait to add to the node.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        """
        path = "%s/traits/%s" % (node_ident, trait)
        return self.update(path, None, http_method='PUT',
                           os_ironic_api_version=os_ironic_api_version,
                           global_request_id=global_request_id)

    def set_traits(self, node_ident, traits, os_ironic_api_version=None,
                   global_request_id=None):
        """Set traits for a node.

        Removes any existing traits and adds the traits passed in to this
        method.

        :param node_ident: node UUID or name.
        :param traits: list of traits to add to the node.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        """
        path = "%s/traits" % node_ident
        body = {'traits': traits}
        return self.update(path, body, http_method='PUT',
                           os_ironic_api_version=os_ironic_api_version,
                           global_request_id=global_request_id)

    def remove_trait(self, node_ident, trait, os_ironic_api_version=None,
                     global_request_id=None):
        """Remove a trait from a node.

        :param node_ident: node UUID or name.
        :param trait: trait to remove from the node.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        """
        path = "%s/traits/%s" % (node_ident, trait)
        return self.delete(path, os_ironic_api_version=os_ironic_api_version,
                           global_request_id=global_request_id)

    def remove_all_traits(self, node_ident, os_ironic_api_version=None,
                          global_request_id=None):
        """Remove all traits from a node.

        :param node_ident: node UUID or name.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        """
        path = "%s/traits" % node_ident
        return self.delete(path, os_ironic_api_version=os_ironic_api_version,
                           global_request_id=global_request_id)

    def get_bios_setting(self, node_ident, name, os_ironic_api_version=None,
                         global_request_id=None):
        """Get a BIOS setting from a node.

        :param node_ident: node UUID or name.
        :param name: BIOS setting name to get from the node.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        """
        path = "%s/bios/%s" % (node_ident, name)
        return self._get_as_dict(
            path, os_ironic_api_version=os_ironic_api_version,
            global_request_id=global_request_id).get(name)

    def list_bios_settings(self, node_ident, detail=False, fields=None,
                           os_ironic_api_version=None, global_request_id=None):
        """List all BIOS settings from a node.

        :param node_ident: node UUID or name.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        :param detail: Optional, boolean whether to return detailed information
                       about bios settings.
        :param fields: Optional, a list with a specified set of fields
                       of the resource to be returned. Can not be used
                       when 'detail' is set.

        """
        if detail and fields:
            raise exc.InvalidAttribute(_("Can't fetch a subset of fields "
                                         "with 'detail' set"))

        filters = utils.common_filters(detail=detail, fields=fields)
        path = "%s/bios" % node_ident

        if filters:
            path += '?' + '&'.join(filters)

        return self._list_primitives(
            self._path(path), 'bios',
            os_ironic_api_version=os_ironic_api_version,
            global_request_id=global_request_id)

    def wait_for_provision_state(self, node_ident, expected_state,
                                 timeout=0,
                                 poll_interval=_DEFAULT_POLL_INTERVAL,
                                 poll_delay_function=None,
                                 fail_on_unexpected_state=True,
                                 os_ironic_api_version=None,
                                 global_request_id=None):
        """Helper function to wait for a node to reach a given state.

        Polls Ironic API in a loop until node gets to a requested state.

        Fails in the following cases:
        * Timeout (if provided) is reached
        * Node's last_error gets set to a non-empty value
        * Unexpected stable state is reached and fail_on_unexpected_state is on
        * Error state is reached (if it's not equal to expected_state)

        :param node_ident: node UUID or name
        :param expected_state: expected final provision state
        :param timeout: timeout in seconds, no timeout if 0
        :param poll_interval: interval in seconds between 2 poll
        :param poll_delay_function: function to use to wait between polls
            (defaults to time.sleep). Should take one argument - delay time
            in seconds. Any exceptions raised inside it will abort the wait.
        :param fail_on_unexpected_state: whether to fail if the nodes
            reaches a different stable state.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :raises: StateTransitionFailed if node reached an error state
        :raises: StateTransitionTimeout on timeout
        """
        expected_state = expected_state.lower()
        timeout_msg = _('Node %(node)s failed to reach state %(state)s in '
                        '%(timeout)s seconds') % {'node': node_ident,
                                                  'state': expected_state,
                                                  'timeout': timeout}

        # TODO(dtantsur): use version negotiation to request API 1.8 and use
        # the "fields" argument to reduce amount of data sent.
        for _count in utils.poll(timeout, poll_interval, poll_delay_function,
                                 timeout_msg):
            node = self.get(
                node_ident, os_ironic_api_version=os_ironic_api_version,
                global_request_id=global_request_id)
            if node.provision_state == expected_state:
                LOG.debug('Node %(node)s reached provision state %(state)s',
                          {'node': node_ident, 'state': expected_state})
                return

            # Note that if expected_state == 'error' we still succeed
            if (node.provision_state == 'error'
                    or node.provision_state.endswith(' failed')):
                raise exc.StateTransitionFailed(
                    _('Node %(node)s failed to reach state %(state)s. '
                      'It\'s in state %(actual)s, and has error: %(error)s') %
                    {'node': node_ident, 'state': expected_state,
                     'actual': node.provision_state, 'error': node.last_error})

            if fail_on_unexpected_state and not node.target_provision_state:
                raise exc.StateTransitionFailed(
                    _('Node %(node)s failed to reach state %(state)s. '
                      'It\'s in unexpected stable state %(actual)s') %
                    {'node': node_ident, 'state': expected_state,
                     'actual': node.provision_state})

            LOG.debug('Still waiting for node %(node)s to reach state '
                      '%(state)s, the current state is %(actual)s',
                      {'node': node_ident, 'state': expected_state,
                       'actual': node.provision_state})



#    Copyright (c) 2017 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import exceptions

from ironicclient.tests.functional.osc.v1 import base


class TestNodeListFields(base.TestCase):
    """Functional tests for "baremetal node list" with --fields."""

    def setUp(self):
        super(TestNodeListFields, self).setUp()
        self.node = self.node_create()

    def _get_table_headers(self, raw_output):
        table = self.parser.table(raw_output)
        return table['headers']

    def test_list_default_fields(self):
        """Test presence of default list table headers."""
        headers = ['UUID', 'Name', 'Instance UUID',
                   'Power State', 'Provisioning State', 'Maintenance']

        nodes_list = self.openstack('baremetal node list')
        nodes_list_headers = self._get_table_headers(nodes_list)

        self.assertEqual(set(headers), set(nodes_list_headers))

    def test_list_minimal_fields(self):
        headers = ['Instance UUID', 'Name', 'UUID']
        fields = ['instance_uuid', 'name', 'uuid']

        node_list = self.openstack(
            'baremetal node list --fields {}'.format(' '.join(fields)))

        nodes_list_headers = self._get_table_headers(node_list)
        self.assertEqual(headers, nodes_list_headers)

    def test_list_no_fields(self):
        command = 'baremetal node list --fields'
        ex_text = 'expected at least one argument'
        self.assertRaisesRegex(exceptions.CommandFailed, ex_text,
                               self.openstack, command)

    def test_list_wrong_field(self):
        command = 'baremetal node list --fields ABC'
        ex_text = 'invalid choice'
        self.assertRaisesRegex(exceptions.CommandFailed, ex_text,
                               self.openstack, command)


class TestNodeShowFields(base.TestCase):
    """Functional tests for "baremetal node show" with --fields."""

    def setUp(self):
        super(TestNodeShowFields, self).setUp()
        self.node = self.node_create()
        self.api_version = '--os-baremetal-api-version 1.20'

    def _get_table_rows(self, raw_output):
        table = self.parser.table(raw_output)
        rows = []
        for row in table['values']:
            rows.append(row[0])
        return rows

    def test_show_default_fields(self):
        rows = ['console_enabled',
                'clean_step',
                'created_at',
                'deploy_step',
                'driver',
                'driver_info',
                'driver_internal_info',
                'extra',
                'inspection_finished_at',
                'inspection_started_at',
                'instance_info',
                'instance_uuid',
                'last_error',
                'maintenance',
                'maintenance_reason',
                'name',
                'power_state',
                'properties',
                'provision_state',
                'provision_updated_at',
                'reservation',
                'target_power_state',
                'target_provision_state',
                'updated_at',
                'uuid']
        node_show = self.openstack('baremetal node show {}'
                                   .format(self.node['uuid']))
        nodes_show_rows = self._get_table_rows(node_show)

        self.assertTrue(set(rows).issubset(set(nodes_show_rows)))

    def test_show_minimal_fields(self):
        rows = [
            'instance_uuid',
            'name',
            'uuid']

        node_show = self.openstack(
            'baremetal node show {} --fields {} {}'.format(
                self.node['uuid'], ' '.join(rows), self.api_version))

        nodes_show_rows = self._get_table_rows(node_show)
        self.assertEqual(set(rows), set(nodes_show_rows))

    def test_show_no_fields(self):
        command = 'baremetal node show {} --fields {}'.format(
            self.node['uuid'], self.api_version)
        ex_text = 'expected at least one argument'
        self.assertRaisesRegex(exceptions.CommandFailed, ex_text,
                               self.openstack, command)

    def test_show_wrong_field(self):
        command = 'baremetal node show {} --fields ABC {}'.format(
            self.node['uuid'], self.api_version)
        ex_text = 'invalid choice'
        self.assertRaisesRegex(exceptions.CommandFailed, ex_text,
                               self.openstack, command)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

import ddt
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from ironicclient.tests.functional.osc.v1 import base


@ddt.ddt
class BaremetalDeployTemplateTests(base.TestCase):
    """Functional tests for baremetal deploy template commands."""

    @staticmethod
    def _get_random_trait():
        return data_utils.rand_name('CUSTOM', '').replace('-', '_')

    def setUp(self):
        super(BaremetalDeployTemplateTests, self).setUp()
        self.steps = json.dumps([{
            'interface': 'bios',
            'step': 'apply_configuration',
            'args': {},
            'priority': 10,
        }])
        name = self._get_random_trait()
        self.template = self.deploy_template_create(
            name, params="--steps '%s'" % self.steps)

    def tearDown(self):
        if self.template is not None:
            self.deploy_template_delete(self.template['uuid'])
        super(BaremetalDeployTemplateTests, self).tearDown()

    def test_list(self):
        """Check baremetal deploy template list command.

        Test steps:
        1) Create baremetal deploy template in setUp.
        2) List baremetal deploy templates.
        3) Check deploy template name and UUID in deploy templates list.
        """
        template_list = self.deploy_template_list()
        self.assertIn(self.template['name'],
                      [template['Name']
                       for template in template_list])
        self.assertIn(self.template['uuid'],
                      [template['UUID']
                       for template in template_list])

    def test_list_long(self):
        """Check baremetal deploy template list --long command

        Test steps:
        1) Create baremetal deploy template in setUp.
        2) List baremetal deploy templates with detail=True.
        3) Check deploy template fields in output.
        """
        template_list = self.deploy_template_list(params='--long')
        template = [template for template in template_list
                    if template['Name'] == self.template['name']][0]
        self.assertEqual(self.template['extra'], template['Extra'])
        self.assertEqual(self.template['name'], template['Name'])
        self.assertEqual(self.template['steps'], template['Steps'])
        self.assertEqual(self.template['uuid'], template['UUID'])

    def test_show(self):
        """Check baremetal deploy template show command with UUID.

        Test steps:
        1) Create baremetal deploy template in setUp.
        2) Show baremetal deploy template calling it by UUID.
        3) Check deploy template fields in output.
        """
        template = self.deploy_template_show(self.template['uuid'])
        self.assertEqual(self.template['extra'], template['extra'])
        self.assertEqual(self.template['name'], template['name'])
        self.assertEqual(self.template['steps'], template['steps'])
        self.assertEqual(self.template['uuid'], template['uuid'])

    def test_delete(self):
        """Check baremetal deploy template delete command.

        Test steps:
        1) Create baremetal deploy template in setUp.
        2) Delete baremetal deploy template by UUID.
        3) Check that deploy template deleted successfully and not in list.
        """
        output = self.deploy_template_delete(self.template['uuid'])
        self.assertIn('Deleted deploy template {0}'.format(
                      self.template['uuid']), output)
        template_list = self.deploy_template_list()
        self.assertNotIn(self.template['name'],
                         [template['Name'] for template in template_list])
        self.assertNotIn(self.template['uuid'],
                         [template['UUID'] for template in template_list])
        self.template = None

    def test_set_steps(self):
        """Check baremetal deploy template set command for steps.

        Test steps:
        1) Create baremetal deploy template in setUp.
        2) Set steps for deploy template.
        3) Check that baremetal deploy template steps were set.
        """
        steps = [{
            'interface': 'bios',
            'step': 'apply_configuration',
            'args': {},
            'priority': 20,
        }]
        self.openstack("baremetal deploy template set --steps '{0}' {1}"
                       .format(json.dumps(steps), self.template['uuid']))

        show_prop = self.deploy_template_show(self.template['uuid'],
                                              fields=['steps'])
        self.assertEqual(steps, show_prop['steps'])

    def test_set_unset(self):
        """Check baremetal deploy template set and unset commands.

        Test steps:
        1) Create baremetal deploy template in setUp.
        2) Set extra data for deploy template.
        3) Check that baremetal deploy template extra data was set.
        4) Unset extra data for deploy template.
        5) Check that baremetal deploy template  extra data was unset.
        """
        extra_key = 'ext'
        extra_value = 'testdata'
        self.openstack(
            'baremetal deploy template set --extra {0}={1} {2}'
            .format(extra_key, extra_value, self.template['uuid']))

        show_prop = self.deploy_template_show(self.template['uuid'],
                                              fields=['extra'])
        self.assertEqual(extra_value, show_prop['extra'][extra_key])

        self.openstack('baremetal deploy template unset --extra {0} {1}'
                       .format(extra_key, self.template['uuid']))

        show_prop = self.deploy_template_show(self.template['uuid'],
                                              fields=['extra'])
        self.assertNotIn(extra_key, show_prop['extra'])

    @ddt.data(
        ('--uuid', '', 'expected one argument'),
        ('--uuid', '!@#$^*&%^', 'Expected UUID for uuid'),
        ('', '', 'the following arguments are required'),
        ('', 'not/a/name', 'does not match'),
        ('', 'foo', 'does not match'),
        ('--steps', '', 'expected one argument'),
        ('--steps', '[]', 'is too short'))
    @ddt.unpack
    def test_create_negative(self, argument, value, ex_text):
        """Check errors on invalid input parameters."""
        base_cmd = 'baremetal deploy template create'
        if argument != '':
            base_cmd += ' %s' % self._get_random_trait()
        if argument != '--steps':
            base_cmd += " --steps '%s'" % self.steps
        command = self.construct_cmd(base_cmd, argument, value)
        self.assertRaisesRegex(exceptions.CommandFailed, ex_text,
                               self.openstack, command)
